@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,14 +27,14 @@
  * ---------------
  * SymbolAxis.java
  * ---------------
- * (C) Copyright 2002-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2002-2007, by Anthony Boulestreau and Contributors.
  *
  * Original Author:  Anthony Boulestreau;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *
  *
- * Changes (from 23-Jun-2001)
- * --------------------------
+ * Changes
+ * -------
  * 29-Mar-2002 : First version (AB);
  * 19-Apr-2002 : Updated formatting and import statements (DG);
  * 21-Jun-2002 : Make change to use the class TickUnit - remove valueToString() 
@@ -70,7 +70,11 @@
  *               symbolicGridPaint --> gridBandPaint, fixed serialization of 
  *               gridBandPaint, renamed symbolicGridLinesVisible --> 
  *               gridBandsVisible, eliminated symbolicGridLineList (DG);
- *
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 02-Feb-2007 : Removed author tags all over JFreeChart sources (DG);
+ * 28-Feb-2007 : Fixed bug 1669302 (tick label overlap) (DG);
+ * 25-Jul-2007 : Added new field for alternate grid band paint (DG);
+ * 
  */
 
 package org.jfree.chart.axis;
@@ -81,6 +85,7 @@ import java.awt.Font;
 import java.awt.Graphics2D;
 import java.awt.Paint;
 import java.awt.Shape;
+import java.awt.Stroke;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.ObjectInputStream;
@@ -104,8 +109,6 @@ import org.jfree.util.PaintUtilities;
 
 /**
  * A standard linear value axis that replaces integer values with symbols.
- *
- * @author Anthony Boulestreau
  */
 public class SymbolAxis extends NumberAxis implements Serializable {
 
@@ -114,22 +117,37 @@ public class SymbolAxis extends NumberAxis implements Serializable {
     
     /** The default grid band paint. */
     public static final Paint DEFAULT_GRID_BAND_PAINT 
-        = new Color(232, 234, 232, 128);
+            = new Color(232, 234, 232, 128);
 
+    /**
+     * The default paint for alternate grid bands.
+     * 
+     * @since 1.0.7
+     */
+    public static final Paint DEFAULT_GRID_BAND_ALTERNATE_PAINT
+            = new Color(0, 0, 0, 0);  // transparent
+    
     /** The list of symbols to display instead of the numeric values. */
     private List symbols;
 
-    /** The paint used to color the grid bands (if the bands are visible). */
-    private transient Paint gridBandPaint;
-
     /** Flag that indicates whether or not grid bands are visible. */
     private boolean gridBandsVisible;
 
+    /** The paint used to color the grid bands (if the bands are visible). */
+    private transient Paint gridBandPaint;
+    
+    /** 
+     * The paint used to fill the alternate grid bands.
+     * 
+     * @since 1.0.7
+     */
+    private transient Paint gridBandAlternatePaint;
+
     /**
      * Constructs a symbol axis, using default attribute values where 
      * necessary.
      *
-     * @param label  the axis label (null permitted).
+     * @param label  the axis label (<code>null</code> permitted).
      * @param sv  the list of symbols to display instead of the numeric
      *            values.
      */
@@ -138,7 +156,7 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         this.symbols = Arrays.asList(sv);
         this.gridBandsVisible = true;
         this.gridBandPaint = DEFAULT_GRID_BAND_PAINT;
-
+        this.gridBandAlternatePaint = DEFAULT_GRID_BAND_ALTERNATE_PAINT;
         setAutoTickUnitSelection(false, false);
         setAutoRangeStickyZero(false);
 
@@ -155,11 +173,40 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         return result;
     }
 
+    /**
+     * Returns <code>true</code> if the grid bands are showing, and
+     * <code>false</code> otherwise.
+     *
+     * @return <code>true</code> if the grid bands are showing, and 
+     *         <code>false</code> otherwise.
+     *         
+     * @see #setGridBandsVisible(boolean)
+     */
+    public boolean isGridBandsVisible() {
+        return this.gridBandsVisible;
+    }
+
+    /**
+     * Sets the visibility of the grid bands and notifies registered
+     * listeners that the axis has been modified.
+     *
+     * @param flag  the new setting.
+     * 
+     * @see #isGridBandsVisible()
+     */
+    public void setGridBandsVisible(boolean flag) {
+        if (this.gridBandsVisible != flag) {
+            this.gridBandsVisible = flag;
+            notifyListeners(new AxisChangeEvent(this));
+        }
+    }
+
     /**
      * Returns the paint used to color the grid bands.
      *
      * @return The grid band paint (never <code>null</code>).
      * 
+     * @see #setGridBandPaint(Paint)
      * @see #isGridBandsVisible()
      */
     public Paint getGridBandPaint() {
@@ -171,6 +218,8 @@ public class SymbolAxis extends NumberAxis implements Serializable {
      * all registered listeners.
      * 
      * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getGridBandPaint()
      */
     public void setGridBandPaint(Paint paint) {
         if (paint == null) {
@@ -179,31 +228,40 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         this.gridBandPaint = paint;
         notifyListeners(new AxisChangeEvent(this));
     }
+
+    /**
+     * Returns the paint used for alternate grid bands.
+     * 
+     * @return The paint (never <code>null</code>).
+     * 
+     * @see #setGridBandAlternatePaint(Paint)
+     * @see #getGridBandPaint()
+     * 
+     * @since 1.0.7
+     */
+    public Paint getGridBandAlternatePaint() {
+        return this.gridBandAlternatePaint;
+    }
     
     /**
-     * Returns <code>true</code> if the grid bands are showing, and
-     * <code>false</code> otherwise.
-     *
-     * @return <code>true</code> if the grid bands are showing, and 
-     *         <code>false</code> otherwise.
+     * Sets the paint used for alternate grid bands and sends a 
+     * {@link AxisChangeEvent} to all registered listeners.
+     * 
+     * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getGridBandAlternatePaint()
+     * @see #setGridBandPaint(Paint)
+     * 
+     * @since 1.0.7
      */
-    public boolean isGridBandsVisible() {
-        return this.gridBandsVisible;
-    }
-
-    /**
-     * Sets the visibility of the grid bands and notifies registered
-     * listeners that the axis has been modified.
-     *
-     * @param flag  the new setting.
-     */
-    public void setGridBandsVisible(boolean flag) {
-        if (this.gridBandsVisible != flag) {
-            this.gridBandsVisible = flag;
-            notifyListeners(new AxisChangeEvent(this));
+    public void setGridBandAlternatePaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
         }
+        this.gridBandAlternatePaint = paint;
+        notifyListeners(new AxisChangeEvent(this));
     }
-
+    
     /**
      * This operation is not supported by this axis.
      *
@@ -319,12 +377,10 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         Rectangle2D band;
         while (iterator.hasNext()) {
             tick = (ValueTick) iterator.next();
-            xx1 = valueToJava2D(
-                tick.getValue() - 0.5d, dataArea, RectangleEdge.BOTTOM
-            );
-            xx2 = valueToJava2D(
-                tick.getValue() + 0.5d, dataArea, RectangleEdge.BOTTOM
-            );
+            xx1 = valueToJava2D(tick.getValue() - 0.5d, dataArea, 
+                    RectangleEdge.BOTTOM);
+            xx2 = valueToJava2D(tick.getValue() + 0.5d, dataArea, 
+                    RectangleEdge.BOTTOM);
             if (currentGridBandIsDark) {
                 g2.setPaint(this.gridBandPaint);
             }
@@ -365,9 +421,9 @@ public class SymbolAxis extends NumberAxis implements Serializable {
 
         //gets the outline stroke width of the plot
         double outlineStrokeWidth;
-        if (getPlot().getOutlineStroke() != null) {
-            outlineStrokeWidth 
-                = ((BasicStroke) getPlot().getOutlineStroke()).getLineWidth();
+        Stroke outlineStroke = getPlot().getOutlineStroke();
+        if (outlineStroke != null && outlineStroke instanceof BasicStroke) {
+            outlineStrokeWidth = ((BasicStroke) outlineStroke).getLineWidth();
         }
         else {
             outlineStrokeWidth = 1d;
@@ -378,20 +434,18 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         Rectangle2D band;
         while (iterator.hasNext()) {
             tick = (ValueTick) iterator.next();
-            yy1 = valueToJava2D(
-                tick.getValue() + 0.5d, plotArea, RectangleEdge.LEFT
-            );
-            yy2 = valueToJava2D(
-                tick.getValue() - 0.5d, plotArea, RectangleEdge.LEFT
-            );
+            yy1 = valueToJava2D(tick.getValue() + 0.5d, plotArea, 
+                    RectangleEdge.LEFT);
+            yy2 = valueToJava2D(tick.getValue() - 0.5d, plotArea, 
+                    RectangleEdge.LEFT);
             if (currentGridBandIsDark) {
                 g2.setPaint(this.gridBandPaint);
             }
             else {
                 g2.setPaint(Color.white);
             }
-            band = new Rectangle2D.Double(xx + outlineStrokeWidth,
-                yy1, plotArea.getMaxX() - xx - outlineStrokeWidth, yy2 - yy1);
+            band = new Rectangle2D.Double(xx + outlineStrokeWidth, yy1, 
+                    plotArea.getMaxX() - xx - outlineStrokeWidth, yy2 - yy1);
             g2.fill(band);
             currentGridBandIsDark = !currentGridBandIsDark;
         }
@@ -538,15 +592,14 @@ public class SymbolAxis extends NumberAxis implements Serializable {
                 }
                 
                 // avoid to draw overlapping tick labels
-                Rectangle2D bounds = TextUtilities.getTextBounds(
-                    tickLabel, g2, g2.getFontMetrics()
-                );
+                Rectangle2D bounds = TextUtilities.getTextBounds(tickLabel, g2, 
+                        g2.getFontMetrics());
                 double tickLabelLength = isVerticalTickLabels() 
-                    ? bounds.getHeight() : bounds.getWidth();
+                        ? bounds.getHeight() : bounds.getWidth();
                 boolean tickLabelsOverlapping = false;
                 if (i > 0) {
                     double avgTickLabelLength = (previousDrawnTickLabelLength 
-                        + tickLabelLength) / 2.0;
+                            + tickLabelLength) / 2.0;
                     if (Math.abs(xx - previousDrawnTickLabelPos) 
                             < avgTickLabelLength) {
                         tickLabelsOverlapping = true;
@@ -584,10 +637,8 @@ public class SymbolAxis extends NumberAxis implements Serializable {
                         rotationAnchor = TextAnchor.TOP_CENTER;
                     }
                 }
-                Tick tick = new NumberTick(
-                    new Double(currentTickValue), tickLabel, anchor, 
-                    rotationAnchor, angle
-                );
+                Tick tick = new NumberTick(new Double(currentTickValue), 
+                        tickLabel, anchor, rotationAnchor, angle);
                 ticks.add(tick);
             }
         }
@@ -635,28 +686,28 @@ public class SymbolAxis extends NumberAxis implements Serializable {
                 }
 
                 // avoid to draw overlapping tick labels
-                Rectangle2D bounds = TextUtilities.getTextBounds(
-                    tickLabel, g2, g2.getFontMetrics()
-                );
+                Rectangle2D bounds = TextUtilities.getTextBounds(tickLabel, g2,
+                        g2.getFontMetrics());
                 double tickLabelLength = isVerticalTickLabels() 
                     ? bounds.getWidth() : bounds.getHeight();
                 boolean tickLabelsOverlapping = false;
                 if (i > 0) {
-                    double avgTickLabelLength = 
-                        (previousDrawnTickLabelLength + tickLabelLength) / 2.0;
+                    double avgTickLabelLength = (previousDrawnTickLabelLength 
+                            + tickLabelLength) / 2.0;
                     if (Math.abs(yy - previousDrawnTickLabelPos) 
                             < avgTickLabelLength) {
                         tickLabelsOverlapping = true;    
                     }
-                    if (tickLabelsOverlapping) {
-                        tickLabel = ""; // don't draw this tick label
-                    }
-                    else {
-                        // remember these values for next comparison
-                        previousDrawnTickLabelPos = yy;
-                        previousDrawnTickLabelLength = tickLabelLength;         
-                    } 
                 }
+                if (tickLabelsOverlapping) {
+                    tickLabel = ""; // don't draw this tick label
+                }
+                else {
+                    // remember these values for next comparison
+                    previousDrawnTickLabelPos = yy;
+                    previousDrawnTickLabelLength = tickLabelLength;         
+                }
+                
                 TextAnchor anchor = null;
                 TextAnchor rotationAnchor = null;
                 double angle = 0.0;
@@ -680,10 +731,8 @@ public class SymbolAxis extends NumberAxis implements Serializable {
                         rotationAnchor = TextAnchor.CENTER_LEFT;
                     }
                 }
-                Tick tick = new NumberTick(
-                    new Double(currentTickValue), tickLabel, anchor, 
-                    rotationAnchor, angle
-                );
+                Tick tick = new NumberTick(new Double(currentTickValue), 
+                        tickLabel, anchor, rotationAnchor, angle);
                 ticks.add(tick);
             }
         }
@@ -733,10 +782,11 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         if (!PaintUtilities.equal(this.gridBandPaint, that.gridBandPaint)) {
             return false;
         }
-        if (!super.equals(obj)) {
+        if (!PaintUtilities.equal(this.gridBandAlternatePaint, 
+                that.gridBandAlternatePaint)) {
             return false;
         }
-        return true;
+        return super.equals(obj);
     }
     
     /**
@@ -749,6 +799,7 @@ public class SymbolAxis extends NumberAxis implements Serializable {
     private void writeObject(ObjectOutputStream stream) throws IOException {
         stream.defaultWriteObject();
         SerialUtilities.writePaint(this.gridBandPaint, stream);
+        SerialUtilities.writePaint(this.gridBandAlternatePaint, stream);
     }
 
     /**
@@ -763,6 +814,7 @@ public class SymbolAxis extends NumberAxis implements Serializable {
         throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.gridBandPaint = SerialUtilities.readPaint(stream);
+        this.gridBandAlternatePaint = SerialUtilities.readPaint(stream);
     }
 
 }
