@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * ------------------------
  * CombinedRangeXYPlot.java
  * ------------------------
- * (C) Copyright 2001-2005, by Bill Kelemen and Contributors.
+ * (C) Copyright 2001-2007, by Bill Kelemen and Contributors.
  *
  * Original Author:  Bill Kelemen;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
@@ -36,8 +36,7 @@
  *                   Kevin Frechette (for ISTI);
  *                   Arnaud Lelievre;
  *                   Nicolas Brodu;
- *
- * $Id: CombinedRangeXYPlot.java,v 1.10.2.1 2005/10/25 20:52:08 mungady Exp $
+ *                   Petr Kubanek (bug 1606205);
  *
  * Changes:
  * --------
@@ -80,7 +79,15 @@
  * 21-Feb-2005 : The getLegendItems() method now returns the fixed legend
  *               items if set (DG);
  * 05-May-2005 : Removed unused draw() method (DG);
- *
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 13-Sep-2006 : Updated API docs (DG);
+ * 06-Feb-2007 : Fixed bug 1606205, draw shared axis after subplots (DG);
+ * 23-Mar-2007 : Reverted previous patch (DG);
+ * 17-Apr-2007 : Added null argument checks to findSubplot() (DG);
+ * 18-Jul-2007 : Fixed bug in removeSubplot (DG);
+ * 27-Nov-2007 : Modified setFixedDomainAxisSpaceForSubplots() so as not to
+ *               trigger change events in subplots (DG);
+ * 
  */
 
 package org.jfree.chart.plot;
@@ -184,6 +191,9 @@ public class CombinedRangeXYPlot extends XYPlot
 
     /**
      * Adds a subplot, with a default 'weight' of 1.
+     * <br><br>
+     * You must ensure that the subplot has a non-null domain axis.  The range
+     * axis for the subplot will be set to <code>null</code>.  
      *
      * @param subplot  the subplot.
      */
@@ -195,6 +205,9 @@ public class CombinedRangeXYPlot extends XYPlot
      * Adds a subplot with a particular weight (greater than or equal to one).  
      * The weight determines how much space is allocated to the subplot 
      * relative to all the other subplots.
+     * <br><br>
+     * You must ensure that the subplot has a non-null domain axis.  The range
+     * axis for the subplot will be set to <code>null</code>.  
      *
      * @param subplot  the subplot.
      * @param weight  the weight (must be 1 or greater).
@@ -241,6 +254,7 @@ public class CombinedRangeXYPlot extends XYPlot
             i++;
         }
         if (position != -1) {
+            this.subplots.remove(position);
             subplot.setParent(null);
             subplot.removeChangeListener(this);
             this.totalWeight -= subplot.getWeight();
@@ -290,9 +304,8 @@ public class CombinedRangeXYPlot extends XYPlot
                 getRangeAxisLocation(), orientation
             );
             if (valueAxis != null) {
-                space = valueAxis.reserveSpace(
-                    g2, this, plotArea, valueEdge, space
-                );
+                space = valueAxis.reserveSpace(g2, this, plotArea, valueEdge, 
+                        space);
             }
         }
         
@@ -319,22 +332,19 @@ public class CombinedRangeXYPlot extends XYPlot
             // calculate sub-plot area
             if (orientation == PlotOrientation.VERTICAL) {
                 double w = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotAreas[i] = new Rectangle2D.Double(
-                    x, y, w, adjustedPlotArea.getHeight()
-                );
+                this.subplotAreas[i] = new Rectangle2D.Double(x, y, w, 
+                        adjustedPlotArea.getHeight());
                 x = x + w + this.gap;
             }
             else if (orientation == PlotOrientation.HORIZONTAL) {
                 double h = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotAreas[i] = new Rectangle2D.Double(
-                    x, y, adjustedPlotArea.getWidth(), h
-                );
+                this.subplotAreas[i] = new Rectangle2D.Double(x, y, 
+                        adjustedPlotArea.getWidth(), h);
                 y = y + h + this.gap;
             }
 
-            AxisSpace subSpace = plot.calculateDomainAxisSpace(
-                g2, this.subplotAreas[i], null
-            );
+            AxisSpace subSpace = plot.calculateDomainAxisSpace(g2, 
+                    this.subplotAreas[i], null);
             space.ensureAtLeast(subSpace);
 
         }
@@ -395,9 +405,8 @@ public class CombinedRangeXYPlot extends XYPlot
                 subplotInfo = new PlotRenderingInfo(info.getOwner());
                 info.addSubplotInfo(subplotInfo);
             }
-            plot.draw(
-                g2, this.subplotAreas[i], anchor, parentState, subplotInfo
-            );
+            plot.draw(g2, this.subplotAreas[i], anchor, parentState, 
+                    subplotInfo);
         }
 
         if (info != null) {
@@ -432,15 +441,25 @@ public class CombinedRangeXYPlot extends XYPlot
      * Multiplies the range on the domain axis/axes by the specified factor.
      *
      * @param factor  the zoom factor.
-     * @param info  the plot rendering info.
-     * @param source  the source point.
+     * @param info  the plot rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
      */
     public void zoomDomainAxes(double factor, PlotRenderingInfo info, 
                                Point2D source) {
+        // delegate 'info' and 'source' argument checks...
         XYPlot subplot = findSubplot(info, source);
         if (subplot != null) {
             subplot.zoomDomainAxes(factor, info, source);
         }
+        else {
+            // if the source point doesn't fall within a subplot, we do the
+            // zoom on all subplots...
+            Iterator iterator = getSubplots().iterator();
+            while (iterator.hasNext()) {
+                subplot = (XYPlot) iterator.next();
+                subplot.zoomDomainAxes(factor, info, source);
+            }
+        }
     }
 
     /**
@@ -448,27 +467,44 @@ public class CombinedRangeXYPlot extends XYPlot
      *
      * @param lowerPercent  the lower bound.
      * @param upperPercent  the upper bound.
-     * @param info  the plot rendering info.
-     * @param source  the source point.
+     * @param info  the plot rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
      */
     public void zoomDomainAxes(double lowerPercent, double upperPercent, 
                                PlotRenderingInfo info, Point2D source) {
+        // delegate 'info' and 'source' argument checks...
         XYPlot subplot = findSubplot(info, source);
         if (subplot != null) {
             subplot.zoomDomainAxes(lowerPercent, upperPercent, info, source);
         }
+        else {
+            // if the source point doesn't fall within a subplot, we do the
+            // zoom on all subplots...
+            Iterator iterator = getSubplots().iterator();
+            while (iterator.hasNext()) {
+                subplot = (XYPlot) iterator.next();
+                subplot.zoomDomainAxes(lowerPercent, upperPercent, info, 
+                        source);
+            }
+        }
     }
 
     /**
      * Returns the subplot (if any) that contains the (x, y) point (specified 
      * in Java2D space).
      * 
-     * @param info  the chart rendering info.
-     * @param source  the source point.
+     * @param info  the chart rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
      * 
      * @return A subplot (possibly <code>null</code>).
      */
     public XYPlot findSubplot(PlotRenderingInfo info, Point2D source) {
+        if (info == null) {
+            throw new IllegalArgumentException("Null 'info' argument.");
+        }
+        if (source == null) {
+            throw new IllegalArgumentException("Null 'source' argument.");
+        }
         XYPlot result = null;
         int subplotIndex = info.getSubplotIndex(source);
         if (subplotIndex >= 0) {
@@ -546,13 +582,11 @@ public class CombinedRangeXYPlot extends XYPlot
      * @param space  the space.
      */
     protected void setFixedDomainAxisSpaceForSubplots(AxisSpace space) {
-
         Iterator iterator = this.subplots.iterator();
         while (iterator.hasNext()) {
             XYPlot plot = (XYPlot) iterator.next();
-            plot.setFixedDomainAxisSpace(space);
+            plot.setFixedDomainAxisSpace(space, false);
         }
-
     }
 
     /**
