@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * ---------
  * Plot.java
  * ---------
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Sylvain Vieujot;
@@ -37,10 +37,8 @@
  *                   Nicolas Brodu;
  *                   Michal Krause;
  *
- * $Id: Plot.java,v 1.18.2.4 2006/06/30 13:04:52 mungady Exp $
- *
- * Changes (from 21-Jun-2001)
- * --------------------------
+ * Changes
+ * -------
  * 21-Jun-2001 : Removed redundant JFreeChart parameter from constructors (DG);
  * 18-Sep-2001 : Updated header info and fixed DOS encoding problem (DG);
  * 19-Oct-2001 : Moved series paint and stroke methods from JFreeChart 
@@ -113,8 +111,14 @@
  * 05-May-2005 : Removed unused draw() method (DG);
  * 06-Jun-2005 : Fixed bugs in equals() method (DG);
  * 01-Sep-2005 : Moved dataAreaRatio from here to ContourPlot (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 30-Jun-2006 : Added background image alpha - see bug report 1514904 (DG);
+ * 05-Sep-2006 : Implemented the MarkerChangeListener interface (DG);
+ * 11-Jan-2007 : Added some argument checks, event notifications, and many
+ *               API doc updates (DG);
+ * 03-Apr-2007 : Made drawBackgroundImage() public (DG);
+ * 07-Jun-2007 : Added new fillBackground() method to handle GradientPaint 
+ *               taking into account orientation (DG);
  *
  */
 
@@ -125,6 +129,7 @@ import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Composite;
 import java.awt.Font;
+import java.awt.GradientPaint;
 import java.awt.Graphics2D;
 import java.awt.Image;
 import java.awt.Paint;
@@ -146,6 +151,8 @@ import org.jfree.chart.axis.AxisLocation;
 import org.jfree.chart.event.AxisChangeEvent;
 import org.jfree.chart.event.AxisChangeListener;
 import org.jfree.chart.event.ChartChangeEventType;
+import org.jfree.chart.event.MarkerChangeEvent;
+import org.jfree.chart.event.MarkerChangeListener;
 import org.jfree.chart.event.PlotChangeEvent;
 import org.jfree.chart.event.PlotChangeListener;
 import org.jfree.data.general.DatasetChangeEvent;
@@ -171,6 +178,7 @@ import org.jfree.util.PublicCloneable;
  */
 public abstract class Plot implements AxisChangeListener,
                                       DatasetChangeListener,
+                                      MarkerChangeListener,
                                       LegendItemSource,
                                       PublicCloneable,
                                       Cloneable,
@@ -233,12 +241,19 @@ public abstract class Plot implements AxisChangeListener,
     /** Amount of blank space around the plot area. */
     private RectangleInsets insets;
 
+    /** 
+     * A flag that controls whether or not the plot outline is drawn. 
+     *
+     * @since 1.0.6
+     */
+    private boolean outlineVisible;
+
     /** The Stroke used to draw an outline around the plot. */
     private transient Stroke outlineStroke;
 
     /** The Paint used to draw an outline around the plot. */
     private transient Paint outlinePaint;
-
+    
     /** An optional color used to fill the plot background. */
     private transient Paint backgroundPaint;
 
@@ -273,6 +288,7 @@ public abstract class Plot implements AxisChangeListener,
         this.backgroundPaint = DEFAULT_BACKGROUND_PAINT;
         this.backgroundAlpha = DEFAULT_BACKGROUND_ALPHA;
         this.backgroundImage = null;
+        this.outlineVisible = true;
         this.outlineStroke = DEFAULT_OUTLINE_STROKE;
         this.outlinePaint = DEFAULT_OUTLINE_PAINT;
         this.foregroundAlpha = DEFAULT_FOREGROUND_ALPHA;
@@ -291,6 +307,8 @@ public abstract class Plot implements AxisChangeListener,
      * Returns the dataset group for the plot (not currently used).
      *
      * @return The dataset group.
+     * 
+     * @see #setDatasetGroup(DatasetGroup)
      */
     public DatasetGroup getDatasetGroup() {
         return this.datasetGroup;
@@ -300,6 +318,8 @@ public abstract class Plot implements AxisChangeListener,
      * Sets the dataset group (not currently used).
      *
      * @param group  the dataset group (<code>null</code> permitted).
+     * 
+     * @see #getDatasetGroup()
      */
     protected void setDatasetGroup(DatasetGroup group) {
         this.datasetGroup = group;
@@ -310,54 +330,83 @@ public abstract class Plot implements AxisChangeListener,
      * <code>null</code>.
      *
      * @return The 'no data' message (<code>null</code> possible).
+     * 
+     * @see #setNoDataMessage(String)
+     * @see #getNoDataMessageFont()
+     * @see #getNoDataMessagePaint()
      */
     public String getNoDataMessage() {
         return this.noDataMessage;
     }
 
     /**
-     * Sets the message that is displayed when the dataset is empty or null.
+     * Sets the message that is displayed when the dataset is empty or 
+     * <code>null</code>, and sends a {@link PlotChangeEvent} to all registered
+     * listeners.
      *
-     * @param message  the message (null permitted).
+     * @param message  the message (<code>null</code> permitted).
+     * 
+     * @see #getNoDataMessage()
      */
     public void setNoDataMessage(String message) {
         this.noDataMessage = message;
+        notifyListeners(new PlotChangeEvent(this));
     }
 
     /**
      * Returns the font used to display the 'no data' message.
      *
-     * @return The font.
+     * @return The font (never <code>null</code>).
+     * 
+     * @see #setNoDataMessageFont(Font)
+     * @see #getNoDataMessage()
      */
     public Font getNoDataMessageFont() {
         return this.noDataMessageFont;
     }
 
     /**
-     * Sets the font used to display the 'no data' message.
+     * Sets the font used to display the 'no data' message and sends a 
+     * {@link PlotChangeEvent} to all registered listeners.
      *
-     * @param font  the font.
+     * @param font  the font (<code>null</code> not permitted).
+     * 
+     * @see #getNoDataMessageFont()
      */
     public void setNoDataMessageFont(Font font) {
+        if (font == null) {
+            throw new IllegalArgumentException("Null 'font' argument.");
+        }
         this.noDataMessageFont = font;
+        notifyListeners(new PlotChangeEvent(this));
     }
 
     /**
      * Returns the paint used to display the 'no data' message.
      *
-     * @return The paint.
+     * @return The paint (never <code>null</code>).
+     * 
+     * @see #setNoDataMessagePaint(Paint)
+     * @see #getNoDataMessage()
      */
     public Paint getNoDataMessagePaint() {
         return this.noDataMessagePaint;
     }
 
     /**
-     * Sets the paint used to display the 'no data' message.
+     * Sets the paint used to display the 'no data' message and sends a 
+     * {@link PlotChangeEvent} to all registered listeners.
      *
-     * @param paint  the paint.
+     * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getNoDataMessagePaint()
      */
     public void setNoDataMessagePaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.noDataMessagePaint = paint;
+        notifyListeners(new PlotChangeEvent(this));
     }
 
     /**
@@ -366,7 +415,8 @@ public abstract class Plot implements AxisChangeListener,
      * Note: this gets used in the chart property editing user interface,
      * but there needs to be a better mechanism for identifying the plot type.
      *
-     * @return A short string describing the plot type.
+     * @return A short string describing the plot type (never 
+     *     <code>null</code>).
      */
     public abstract String getPlotType();
 
@@ -375,15 +425,21 @@ public abstract class Plot implements AxisChangeListener,
      * of a combined plot).
      *
      * @return The parent plot.
+     * 
+     * @see #setParent(Plot)
+     * @see #getRootPlot()
      */
     public Plot getParent() {
         return this.parent;
     }
 
     /**
-     * Sets the parent plot.
+     * Sets the parent plot.  This method is intended for internal use, you 
+     * shouldn't need to call it directly.
      *
-     * @param parent  the parent plot.
+     * @param parent  the parent plot (<code>null</code> permitted).
+     * 
+     * @see #getParent()
      */
     public void setParent(Plot parent) {
         this.parent = parent;
@@ -393,6 +449,8 @@ public abstract class Plot implements AxisChangeListener,
      * Returns the root plot.
      *
      * @return The root plot.
+     * 
+     * @see #getParent()
      */
     public Plot getRootPlot() {
 
@@ -407,10 +465,14 @@ public abstract class Plot implements AxisChangeListener,
     }
 
     /**
-     * Returns true if this plot is part of a combined plot structure.
+     * Returns <code>true</code> if this plot is part of a combined plot 
+     * structure (that is, {@link #getParent()} returns a non-<code>null</code>
+     * value), and <code>false</code> otherwise.
      *
      * @return <code>true</code> if this plot is part of a combined plot 
      *         structure.
+     *         
+     * @see #getParent()
      */
     public boolean isSubplot() {
         return (getParent() != null);
@@ -420,6 +482,8 @@ public abstract class Plot implements AxisChangeListener,
      * Returns the insets for the plot area.
      *
      * @return The insets (never <code>null</code>).
+     * 
+     * @see #setInsets(RectangleInsets)
      */
     public RectangleInsets getInsets() {
         return this.insets;
@@ -430,6 +494,9 @@ public abstract class Plot implements AxisChangeListener,
      * all registered listeners.
      *
      * @param insets  the new insets (<code>null</code> not permitted).
+     * 
+     * @see #getInsets()
+     * @see #setInsets(RectangleInsets, boolean)
      */
     public void setInsets(RectangleInsets insets) {
         setInsets(insets, true);
@@ -442,6 +509,9 @@ public abstract class Plot implements AxisChangeListener,
      * @param insets  the new insets (<code>null</code> not permitted).
      * @param notify  a flag that controls whether the registered listeners are
      *                notified.
+     *                
+     * @see #getInsets()
+     * @see #setInsets(RectangleInsets)
      */
     public void setInsets(RectangleInsets insets, boolean notify) {
         if (insets == null) {
@@ -460,6 +530,8 @@ public abstract class Plot implements AxisChangeListener,
      * Returns the background color of the plot area.
      *
      * @return The paint (possibly <code>null</code>).
+     * 
+     * @see #setBackgroundPaint(Paint)
      */
     public Paint getBackgroundPaint() {
         return this.backgroundPaint;
@@ -470,6 +542,8 @@ public abstract class Plot implements AxisChangeListener,
      * {@link PlotChangeEvent} to all registered listeners.
      *
      * @param paint  the paint (<code>null</code> permitted).
+     * 
+     * @see #getBackgroundPaint()
      */
     public void setBackgroundPaint(Paint paint) {
 
@@ -495,6 +569,8 @@ public abstract class Plot implements AxisChangeListener,
      * Returns the alpha transparency of the plot area background.
      *
      * @return The alpha transparency.
+     * 
+     * @see #setBackgroundAlpha(float)
      */
     public float getBackgroundAlpha() {
         return this.backgroundAlpha;
@@ -504,21 +580,23 @@ public abstract class Plot implements AxisChangeListener,
      * Sets the alpha transparency of the plot area background, and notifies
      * registered listeners that the plot has been modified.
      *
-     * @param alpha the new alpha value.
+     * @param alpha the new alpha value (in the range 0.0f to 1.0f).
+     * 
+     * @see #getBackgroundAlpha()
      */
     public void setBackgroundAlpha(float alpha) {
-
         if (this.backgroundAlpha != alpha) {
             this.backgroundAlpha = alpha;
             notifyListeners(new PlotChangeEvent(this));
         }
-
     }
 
     /**
      * Returns the drawing supplier for the plot.
      *
      * @return The drawing supplier (possibly <code>null</code>).
+     * 
+     * @see #setDrawingSupplier(DrawingSupplier)
      */
     public DrawingSupplier getDrawingSupplier() {
         DrawingSupplier result = null;
@@ -539,6 +617,8 @@ public abstract class Plot implements AxisChangeListener,
      * that the plot's renderer(s) can use to populate its (their) tables.
      *
      * @param supplier  the new supplier.
+     * 
+     * @see #getDrawingSupplier()
      */
     public void setDrawingSupplier(DrawingSupplier supplier) {
         this.drawingSupplier = supplier;
@@ -550,15 +630,20 @@ public abstract class Plot implements AxisChangeListener,
      * area.
      *
      * @return The image (possibly <code>null</code>).
+     * 
+     * @see #setBackgroundImage(Image)
      */
     public Image getBackgroundImage() {
         return this.backgroundImage;
     }
 
     /**
-     * Sets the background image for the plot.
+     * Sets the background image for the plot and sends a 
+     * {@link PlotChangeEvent} to all registered listeners.
      *
      * @param image  the image (<code>null</code> permitted).
+     * 
+     * @see #getBackgroundImage()
      */
     public void setBackgroundImage(Image image) {
         this.backgroundImage = image;
@@ -571,6 +656,8 @@ public abstract class Plot implements AxisChangeListener,
      * library.
      *
      * @return The alignment.
+     * 
+     * @see #setBackgroundImageAlignment(int)
      */
     public int getBackgroundImageAlignment() {
         return this.backgroundImageAlignment;
@@ -583,6 +670,8 @@ public abstract class Plot implements AxisChangeListener,
      * class library.
      *
      * @param alignment  the alignment.
+     * 
+     * @see #getBackgroundImageAlignment()
      */
     public void setBackgroundImageAlignment(int alignment) {
         if (this.backgroundImageAlignment != alignment) {
@@ -615,8 +704,7 @@ public abstract class Plot implements AxisChangeListener,
      *     
      * @see #getBackgroundImageAlpha()
      */
-    public void setBackgroundImageAlpha(float alpha)
-    {
+    public void setBackgroundImageAlpha(float alpha) {
         if (alpha < 0.0f || alpha > 1.0f)
             throw new IllegalArgumentException(
                     "The 'alpha' value must be in the range 0.0f to 1.0f.");
@@ -626,10 +714,44 @@ public abstract class Plot implements AxisChangeListener,
         }
     }
     
+    /**
+     * Returns the flag that controls whether or not the plot outline is
+     * drawn.  The default value is <code>true</code>.  Note that for 
+     * historical reasons, the plot's outline paint and stroke can take on
+     * <code>null</code> values, in which case the outline will not be drawn
+     * even if this flag is set to <code>true</code>.
+     * 
+     * @return The outline visibility flag.
+     * 
+     * @since 1.0.6
+     * 
+     * @see #setOutlineVisible(boolean)
+     */
+    public boolean isOutlineVisible() {
+        return this.outlineVisible;    
+    }
+    
+    /**
+     * Sets the flag that controls whether or not the plot's outline is
+     * drawn, and sends a {@link PlotChangeEvent} to all registered listeners.
+     * 
+     * @param visible  the new flag value.
+     * 
+     * @since 1.0.6
+     * 
+     * @see #isOutlineVisible()
+     */
+    public void setOutlineVisible(boolean visible) {
+        this.outlineVisible = visible;
+        notifyListeners(new PlotChangeEvent(this));
+    }
+    
     /**
      * Returns the stroke used to outline the plot area.
      *
      * @return The stroke (possibly <code>null</code>).
+     * 
+     * @see #setOutlineStroke(Stroke)
      */
     public Stroke getOutlineStroke() {
         return this.outlineStroke;
@@ -638,12 +760,13 @@ public abstract class Plot implements AxisChangeListener,
     /**
      * Sets the stroke used to outline the plot area and sends a 
      * {@link PlotChangeEvent} to all registered listeners. If you set this 
-     * attribute to <code>null<.code>, no outline will be drawn.
+     * attribute to <code>null</code>, no outline will be drawn.
      *
      * @param stroke  the stroke (<code>null</code> permitted).
+     * 
+     * @see #getOutlineStroke()
      */
     public void setOutlineStroke(Stroke stroke) {
-
         if (stroke == null) {
             if (this.outlineStroke != null) {
                 this.outlineStroke = null;
@@ -659,13 +782,14 @@ public abstract class Plot implements AxisChangeListener,
             this.outlineStroke = stroke;
             notifyListeners(new PlotChangeEvent(this));
         }
-
     }
 
     /**
      * Returns the color used to draw the outline of the plot area.
      *
      * @return The color (possibly <code>null<code>).
+     * 
+     * @see #setOutlinePaint(Paint)
      */
     public Paint getOutlinePaint() {
         return this.outlinePaint;
@@ -677,9 +801,10 @@ public abstract class Plot implements AxisChangeListener,
      * attribute to <code>null</code>, no outline will be drawn.
      *
      * @param paint  the paint (<code>null</code> permitted).
+     * 
+     * @see #getOutlinePaint()
      */
     public void setOutlinePaint(Paint paint) {
-
         if (paint == null) {
             if (this.outlinePaint != null) {
                 this.outlinePaint = null;
@@ -695,30 +820,32 @@ public abstract class Plot implements AxisChangeListener,
             this.outlinePaint = paint;
             notifyListeners(new PlotChangeEvent(this));
         }
-
     }
 
     /**
      * Returns the alpha-transparency for the plot foreground.
      *
      * @return The alpha-transparency.
+     * 
+     * @see #setForegroundAlpha(float)
      */
     public float getForegroundAlpha() {
         return this.foregroundAlpha;
     }
 
     /**
-     * Sets the alpha-transparency for the plot.
+     * Sets the alpha-transparency for the plot and sends a 
+     * {@link PlotChangeEvent} to all registered listeners.
      *
      * @param alpha  the new alpha transparency.
+     * 
+     * @see #getForegroundAlpha()
      */
     public void setForegroundAlpha(float alpha) {
-
         if (this.foregroundAlpha != alpha) {
             this.foregroundAlpha = alpha;
             notifyListeners(new PlotChangeEvent(this));
         }
-
     }
 
     /**
@@ -736,6 +863,8 @@ public abstract class Plot implements AxisChangeListener,
      * Registers an object for notification of changes to the plot.
      *
      * @param listener  the object to be registered.
+     * 
+     * @see #removeChangeListener(PlotChangeListener)
      */
     public void addChangeListener(PlotChangeListener listener) {
         this.listenerList.add(PlotChangeListener.class, listener);
@@ -745,6 +874,8 @@ public abstract class Plot implements AxisChangeListener,
      * Unregisters an object for notification of changes to the plot.
      *
      * @param listener  the object to be unregistered.
+     * 
+     * @see #addChangeListener(PlotChangeListener)
      */
     public void removeChangeListener(PlotChangeListener listener) {
         this.listenerList.remove(PlotChangeListener.class, listener);
@@ -756,14 +887,12 @@ public abstract class Plot implements AxisChangeListener,
      * @param event  information about the change event.
      */
     public void notifyListeners(PlotChangeEvent event) {
-
         Object[] listeners = this.listenerList.getListenerList();
         for (int i = listeners.length - 2; i >= 0; i -= 2) {
             if (listeners[i] == PlotChangeListener.class) {
                 ((PlotChangeListener) listeners[i + 1]).plotChanged(event);
             }
         }
-
     }
 
     /**
@@ -797,6 +926,8 @@ public abstract class Plot implements AxisChangeListener,
      * @param area  the area within which the plot should be drawn.
      */
     public void drawBackground(Graphics2D g2, Rectangle2D area) {
+        // some subclasses override this method completely, so don't put 
+        // anything here that *must* be done
         fillBackground(g2, area);
         drawBackgroundImage(g2, area);
     }
@@ -806,19 +937,57 @@ public abstract class Plot implements AxisChangeListener,
      * 
      * @param g2  the graphics device.
      * @param area  the area.
+     * 
+     * @see #getBackgroundPaint()
+     * @see #getBackgroundAlpha()
+     * @see #fillBackground(Graphics2D, Rectangle2D, PlotOrientation)
      */
     protected void fillBackground(Graphics2D g2, Rectangle2D area) {
-        if (this.backgroundPaint != null) {
-            Composite originalComposite = g2.getComposite();
-            g2.setComposite(
-                AlphaComposite.getInstance(
-                    AlphaComposite.SRC_OVER, this.backgroundAlpha
-                )
-            );
-            g2.setPaint(this.backgroundPaint);
-            g2.fill(area);
-            g2.setComposite(originalComposite);
-        }        
+        fillBackground(g2, area, PlotOrientation.VERTICAL);
+    }
+    
+    /**
+     * Fills the specified area with the background paint.  If the background
+     * paint is an instance of <code>GradientPaint</code>, the gradient will
+     * run in the direction suggested by the plot's orientation.
+     * 
+     * @param g2  the graphics target.
+     * @param area  the plot area.
+     * @param orientation  the plot orientation (<code>null</code> not 
+     *         permitted).
+     * 
+     * @since 1.0.6
+     */
+    protected void fillBackground(Graphics2D g2, Rectangle2D area, 
+            PlotOrientation orientation) {
+        if (orientation == null) {
+            throw new IllegalArgumentException("Null 'orientation' argument.");
+        }
+        if (this.backgroundPaint == null) {
+            return;
+        }
+        Paint p = this.backgroundPaint;
+        if (p instanceof GradientPaint) {
+            GradientPaint gp = (GradientPaint) p;
+            if (orientation == PlotOrientation.VERTICAL) {
+                p = new GradientPaint((float) area.getCenterX(), 
+                        (float) area.getMaxY(), gp.getColor1(), 
+                        (float) area.getCenterX(), (float) area.getMinY(), 
+                        gp.getColor2());
+            }
+            else if (orientation == PlotOrientation.HORIZONTAL) {
+                p = new GradientPaint((float) area.getMinX(), 
+                        (float) area.getCenterY(), gp.getColor1(), 
+                        (float) area.getMaxX(), (float) area.getCenterY(), 
+                        gp.getColor2());
+            }
+        }            
+        Composite originalComposite = g2.getComposite();
+        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
+                this.backgroundAlpha));
+        g2.setPaint(p);
+        g2.fill(area);
+        g2.setComposite(originalComposite);        
     }
     
     /**
@@ -832,7 +1001,7 @@ public abstract class Plot implements AxisChangeListener,
      * @see #getBackgroundImageAlignment()
      * @see #getBackgroundImageAlpha()
      */
-    protected void drawBackgroundImage(Graphics2D g2, Rectangle2D area) {
+    public void drawBackgroundImage(Graphics2D g2, Rectangle2D area) {
         if (this.backgroundImage != null) {
             Composite originalComposite = g2.getComposite();
             g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
@@ -858,6 +1027,9 @@ public abstract class Plot implements AxisChangeListener,
      * @param area  the area within which the plot should be drawn.
      */
     public void drawOutline(Graphics2D g2, Rectangle2D area) {
+        if (!this.outlineVisible) {
+            return;
+        }
         if ((this.outlineStroke != null) && (this.outlinePaint != null)) {
             g2.setStroke(this.outlineStroke);
             g2.setPaint(this.outlinePaint);
@@ -872,7 +1044,6 @@ public abstract class Plot implements AxisChangeListener,
      * @param area  the area within which the plot should be drawn.
      */
     protected void drawNoDataMessage(Graphics2D g2, Rectangle2D area) {
-
         Shape savedClip = g2.getClip();
         g2.clip(area);
         String message = this.noDataMessage;
@@ -880,17 +1051,13 @@ public abstract class Plot implements AxisChangeListener,
             g2.setFont(this.noDataMessageFont);
             g2.setPaint(this.noDataMessagePaint);
             TextBlock block = TextUtilities.createTextBlock(
-                this.noDataMessage, this.noDataMessageFont, 
-                this.noDataMessagePaint, 
-                0.9f * (float) area.getWidth(), new G2TextMeasurer(g2)
-            );
-            block.draw(
-                g2, (float) area.getCenterX(), (float) area.getCenterY(), 
-                TextBlockAnchor.CENTER
-            );
+                    this.noDataMessage, this.noDataMessageFont, 
+                    this.noDataMessagePaint, 0.9f * (float) area.getWidth(), 
+                    new G2TextMeasurer(g2));
+            block.draw(g2, (float) area.getCenterX(), 
+                    (float) area.getCenterY(), TextBlockAnchor.CENTER);
         }
         g2.setClip(savedClip);
-
     }
 
     /**
@@ -939,6 +1106,18 @@ public abstract class Plot implements AxisChangeListener,
         newEvent.setType(ChartChangeEventType.DATASET_UPDATED);
         notifyListeners(newEvent);
     }
+    
+    /**
+     * Receives notification of a change to a marker that is assigned to the
+     * plot.
+     * 
+     * @param event  the event.
+     * 
+     * @since 1.0.3
+     */
+    public void markerChanged(MarkerChangeEvent event) {
+        notifyListeners(new PlotChangeEvent(this));
+    }
 
     /**
      * Adjusts the supplied x-value.
@@ -1011,14 +1190,16 @@ public abstract class Plot implements AxisChangeListener,
         )) {
             return false;
         }
-        if (!PaintUtilities.equal(
-            this.noDataMessagePaint, that.noDataMessagePaint
-        )) {
+        if (!PaintUtilities.equal(this.noDataMessagePaint, 
+                that.noDataMessagePaint)) {
             return false;
         }
         if (!ObjectUtilities.equal(this.insets, that.insets)) {
             return false;
         }
+        if (this.outlineVisible != that.outlineVisible) {
+            return false;
+        }
         if (!ObjectUtilities.equal(this.outlineStroke, that.outlineStroke)) {
             return false;
         }
@@ -1028,9 +1209,8 @@ public abstract class Plot implements AxisChangeListener,
         if (!PaintUtilities.equal(this.backgroundPaint, that.backgroundPaint)) {
             return false;
         }
-        if (!ObjectUtilities.equal(
-            this.backgroundImage, that.backgroundImage
-        )) {
+        if (!ObjectUtilities.equal(this.backgroundImage, 
+                that.backgroundImage)) {
             return false;
         }
         if (this.backgroundImageAlignment != that.backgroundImageAlignment) {
@@ -1234,4 +1414,3 @@ public abstract class Plot implements AxisChangeListener,
     }
 
 }
-
