@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,13 +27,11 @@
  * --------------
  * PolarPlot.java
  * --------------
- * (C) Copyright 2004, 2005, by Solution Engineering, Inc. and Contributors.
+ * (C) Copyright 2004-2007, by Solution Engineering, Inc. and Contributors.
  *
  * Original Author:  Daniel Bridenbecker, Solution Engineering, Inc.;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *
- * $Id: PolarPlot.java,v 1.13.2.3 2005/10/25 20:52:08 mungady Exp $
- *
  * Changes
  * -------
  * 19-Jan-2004 : Version 1, contributed by DB with minor changes by DG (DG);
@@ -41,11 +39,16 @@
  * 05-May-2005 : Updated draw() method parameters (DG);
  * 09-Jun-2005 : Fixed getDataRange() and equals() methods (DG);
  * 25-Oct-2005 : Implemented Zoomable (DG);
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 07-Feb-2007 : Fixed bug 1599761, data value less than axis minimum (DG);
+ * 21-Mar-2007 : Fixed serialization bug (DG);
+ * 24-Sep-2007 : Implemented new zooming methods (DG);
  *
  */
 
 package org.jfree.chart.plot;
 
+
 import java.awt.AlphaComposite;
 import java.awt.BasicStroke;
 import java.awt.Color;
@@ -92,15 +95,10 @@ import org.jfree.util.PaintUtilities;
 
 /**
  * Plots data that is in (theta, radius) pairs where
- * theta equal to zero is due north and and increases clockwise.
- *
- * @author Daniel Bridenbecker, Solution Engineering, Inc.
+ * theta equal to zero is due north and increases clockwise.
  */
-public class PolarPlot extends Plot implements ValueAxisPlot, 
-                                               Zoomable,
-                                               RendererChangeListener, 
-                                               Cloneable, 
-                                               Serializable {
+public class PolarPlot extends Plot implements ValueAxisPlot, Zoomable,
+        RendererChangeListener, Cloneable, Serializable {
    
     /** For serialization. */
     private static final long serialVersionUID = 3794383185924179525L;
@@ -112,11 +110,9 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     private static final double ANNOTATION_MARGIN = 7.0;
    
     /** The default grid line stroke. */
-    public static final Stroke DEFAULT_GRIDLINE_STROKE 
-        = new BasicStroke(
+    public static final Stroke DEFAULT_GRIDLINE_STROKE = new BasicStroke(
             0.5f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 
-            0.0f, new float[]{2.0f, 2.0f}, 0.0f
-          );
+            0.0f, new float[]{2.0f, 2.0f}, 0.0f);
    
     /** The default grid line paint. */
     public static final Paint DEFAULT_GRIDLINE_PAINT = Color.gray;
@@ -125,9 +121,6 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     protected static ResourceBundle localizationResources 
         = ResourceBundle.getBundle("org.jfree.chart.plot.LocalizationBundle");
    
-    // ------------------------
-    // --- Member Variables ---
-    // ------------------------
     /** The angles that are marked with gridlines. */
     private List angleTicks;
     
@@ -150,7 +143,7 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     private Font angleLabelFont = new Font("SansSerif", Font.PLAIN, 12);
     
     /** The paint used to display the angle labels. */
-    private Paint angleLabelPaint = Color.black;
+    private transient Paint angleLabelPaint = Color.black;
     
     /** A flag that controls whether the angular grid-lines are visible. */
     private boolean angleGridlinesVisible;
@@ -173,9 +166,6 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     /** The annotations for the plot. */
     private List cornerTextItems = new ArrayList();
    
-    // --------------------
-    // --- Constructors ---
-    // --------------------
     /**
      * Default constructor.
      */
@@ -241,9 +231,12 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     }
    
     /**
-     * Add text to be displayed in the lower right hand corner.
+     * Add text to be displayed in the lower right hand corner and sends a 
+     * {@link PlotChangeEvent} to all registered listeners.
      * 
      * @param text  the text to display (<code>null</code> not permitted).
+     * 
+     * @see #removeCornerTextItem(String)
      */
     public void addCornerTextItem(String text) {
         if (text == null) {
@@ -254,9 +247,12 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     }
    
     /**
-     * Remove the given text from the list of corner text items.
+     * Remove the given text from the list of corner text items and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      * 
      * @param text  the text to remove (<code>null</code> ignored).
+     * 
+     * @see #addCornerTextItem(String)
      */
     public void removeCornerTextItem(String text) {
         boolean removed = this.cornerTextItems.remove(text);
@@ -266,7 +262,11 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     }
    
     /**
-     * Clear the list of corner text items.
+     * Clear the list of corner text items and sends a {@link PlotChangeEvent}
+     * to all registered listeners.
+     * 
+     * @see #addCornerTextItem(String)
+     * @see #removeCornerTextItem(String)
      */
     public void clearCornerTextItems() {
         if (this.cornerTextItems.size() > 0) {
@@ -287,7 +287,9 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     /**
      * Returns the axis for the plot.
      *
-     * @return The radius axis.
+     * @return The radius axis (possibly <code>null</code>).
+     * 
+     * @see #setAxis(ValueAxis)
      */
     public ValueAxis getAxis() {
         return this.axis;
@@ -321,6 +323,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the primary dataset for the plot.
      *
      * @return The primary dataset (possibly <code>null</code>).
+     * 
+     * @see #setDataset(XYDataset)
      */
     public XYDataset getDataset() {
         return this.dataset;
@@ -331,6 +335,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * is one.
      *
      * @param dataset  the dataset (<code>null</code> permitted).
+     * 
+     * @see #getDataset()
      */
     public void setDataset(XYDataset dataset) {
         // if there is an existing dataset, remove the plot from the list of 
@@ -356,6 +362,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the item renderer.
      *
      * @return The renderer (possibly <code>null</code>).
+     * 
+     * @see #setRenderer(PolarItemRenderer)
      */
     public PolarItemRenderer getRenderer() {
         return this.renderer;
@@ -368,6 +376,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * If the renderer is set to <code>null</code>, no chart will be drawn.
      *
      * @param renderer  the new renderer (<code>null</code> permitted).
+     * 
+     * @see #getRenderer()
      */
     public void setRenderer(PolarItemRenderer renderer) {
         if (this.renderer != null) {
@@ -386,6 +396,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns a flag that controls whether or not the angle labels are visible.
      * 
      * @return A boolean.
+     * 
+     * @see #setAngleLabelsVisible(boolean)
      */
     public boolean isAngleLabelsVisible() {
         return this.angleLabelsVisible;
@@ -396,6 +408,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * and sends a {@link PlotChangeEvent} to all registered listeners.
      * 
      * @param visible  the flag.
+     * 
+     * @see #isAngleLabelsVisible()
      */
     public void setAngleLabelsVisible(boolean visible) {
         if (this.angleLabelsVisible != visible) {
@@ -408,6 +422,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the font used to display the angle labels.
      * 
      * @return A font (never <code>null</code>).
+     * 
+     * @see #setAngleLabelFont(Font)
      */
     public Font getAngleLabelFont() {
         return this.angleLabelFont;
@@ -418,6 +434,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * {@link PlotChangeEvent} to all registered listeners.
      * 
      * @param font  the font (<code>null</code> not permitted).
+     * 
+     * @see #getAngleLabelFont()
      */
     public void setAngleLabelFont(Font font) {
         if (font == null) {
@@ -430,7 +448,9 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     /**
      * Returns the paint used to display the angle labels.
      * 
-     * @return A paint.
+     * @return A paint (never <code>null</code>).
+     * 
+     * @see #setAngleLabelPaint(Paint)
      */
     public Paint getAngleLabelPaint() {
         return this.angleLabelPaint;
@@ -440,9 +460,12 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Sets the paint used to display the angle labels and sends a 
      * {@link PlotChangeEvent} to all registered listeners.
      * 
-     * @param paint  the paint.
+     * @param paint  the paint (<code>null</code> not permitted).
      */
     public void setAngleLabelPaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.angleLabelPaint = paint;
         notifyListeners(new PlotChangeEvent(this));
     }
@@ -452,6 +475,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * <code>false<code> otherwise.
      *
      * @return <code>true</code> or <code>false</code>.
+     * 
+     * @see #setAngleGridlinesVisible(boolean)
      */
     public boolean isAngleGridlinesVisible() {
         return this.angleGridlinesVisible;
@@ -465,6 +490,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * registered listeners.
      *
      * @param visible  the new value of the flag.
+     * 
+     * @see #isAngleGridlinesVisible()
      */
     public void setAngleGridlinesVisible(boolean visible) {
         if (this.angleGridlinesVisible != visible) {
@@ -477,18 +504,23 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the stroke for the grid-lines (if any) plotted against the 
      * angular axis.
      *
-     * @return The stroke.
+     * @return The stroke (possibly <code>null</code>).
+     * 
+     * @see #setAngleGridlineStroke(Stroke)
      */
     public Stroke getAngleGridlineStroke() {
         return this.angleGridlineStroke;
     }
     
     /**
-     * Sets the stroke for the grid lines plotted against the angular axis.
+     * Sets the stroke for the grid lines plotted against the angular axis and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      * <p>
      * If you set this to <code>null</code>, no grid lines will be drawn.
      *
      * @param stroke  the stroke (<code>null</code> permitted).
+     * 
+     * @see #getAngleGridlineStroke()
      */
     public void setAngleGridlineStroke(Stroke stroke) {
         this.angleGridlineStroke = stroke;
@@ -499,7 +531,9 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the paint for the grid lines (if any) plotted against the 
      * angular axis.
      *
-     * @return The paint.
+     * @return The paint (possibly <code>null</code>).
+     * 
+     * @see #setAngleGridlinePaint(Paint)
      */
     public Paint getAngleGridlinePaint() {
         return this.angleGridlinePaint;
@@ -511,6 +545,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * If you set this to <code>null</code>, no grid lines will be drawn.
      *
      * @param paint  the paint (<code>null</code> permitted).
+     * 
+     * @see #getAngleGridlinePaint()
      */
     public void setAngleGridlinePaint(Paint paint) {
         this.angleGridlinePaint = paint;
@@ -522,6 +558,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * <code>false<code> otherwise.
      *
      * @return <code>true</code> or <code>false</code>.
+     * 
+     * @see #setRadiusGridlinesVisible(boolean)
      */
     public boolean isRadiusGridlinesVisible() {
         return this.radiusGridlinesVisible;
@@ -535,6 +573,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * registered listeners.
      *
      * @param visible  the new value of the flag.
+     * 
+     * @see #isRadiusGridlinesVisible()
      */
     public void setRadiusGridlinesVisible(boolean visible) {
         if (this.radiusGridlinesVisible != visible) {
@@ -547,18 +587,23 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the stroke for the grid lines (if any) plotted against the 
      * radius axis.
      *
-     * @return The stroke.
+     * @return The stroke (possibly <code>null</code>).
+     * 
+     * @see #setRadiusGridlineStroke(Stroke)
      */
     public Stroke getRadiusGridlineStroke() {
         return this.radiusGridlineStroke;
     }
     
     /**
-     * Sets the stroke for the grid lines plotted against the radius axis.
+     * Sets the stroke for the grid lines plotted against the radius axis and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      * <p>
      * If you set this to <code>null</code>, no grid lines will be drawn.
      *
      * @param stroke  the stroke (<code>null</code> permitted).
+     * 
+     * @see #getRadiusGridlineStroke()
      */
     public void setRadiusGridlineStroke(Stroke stroke) {
         this.radiusGridlineStroke = stroke;
@@ -569,18 +614,23 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Returns the paint for the grid lines (if any) plotted against the radius
      * axis.
      *
-     * @return The paint.
+     * @return The paint (possibly <code>null</code>).
+     * 
+     * @see #setRadiusGridlinePaint(Paint)
      */
     public Paint getRadiusGridlinePaint() {
         return this.radiusGridlinePaint;
     }
     
     /**
-     * Sets the paint for the grid lines plotted against the radius axis.
+     * Sets the paint for the grid lines plotted against the radius axis and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      * <p>
      * If you set this to <code>null</code>, no grid lines will be drawn.
      *
      * @param paint  the paint (<code>null</code> permitted).
+     * 
+     * @see #getRadiusGridlinePaint()
      */
     public void setRadiusGridlinePaint(Paint paint) {
         this.radiusGridlinePaint = paint;
@@ -591,8 +641,7 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      * Draws the plot on a Java 2D graphics device (such as the screen or a 
      * printer).
      * <P>
-     * This plot relies on an 
-     * {@link org.jfree.chart.renderer.DefaultPolarItemRenderer} to draw each 
+     * This plot relies on a {@link PolarItemRenderer} to draw each 
      * item in the plot.  This allows the visual representation of the data to 
      * be changed easily.
      * <P>
@@ -637,23 +686,18 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
        
         // draw the plot background and axes...
         drawBackground(g2, dataArea);
-        double h = Math.min(
-            dataArea.getWidth() / 2.0, dataArea.getHeight() / 2.0
-        ) - MARGIN;
-        Rectangle2D quadrant = new Rectangle2D.Double(
-            dataArea.getCenterX(), dataArea.getCenterY(), h, h
-        );
+        double h = Math.min(dataArea.getWidth() / 2.0, 
+                dataArea.getHeight() / 2.0) - MARGIN;
+        Rectangle2D quadrant = new Rectangle2D.Double(dataArea.getCenterX(), 
+                dataArea.getCenterY(), h, h);
         AxisState state = drawAxis(g2, area, quadrant);
         if (this.renderer != null) {
             Shape originalClip = g2.getClip();
             Composite originalComposite = g2.getComposite();
           
             g2.clip(dataArea);
-            g2.setComposite(
-                AlphaComposite.getInstance(
-                    AlphaComposite.SRC_OVER, getForegroundAlpha()
-                )
-            );
+            g2.setComposite(AlphaComposite.getInstance(
+                    AlphaComposite.SRC_OVER, getForegroundAlpha()));
           
             drawGridlines(g2, dataArea, this.angleTicks, state.getTicks());
           
@@ -700,9 +744,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         x += ANNOTATION_MARGIN;
         for (Iterator it = this.cornerTextItems.iterator(); it.hasNext();) {
             String msg = (String) it.next();
-            Rectangle2D bounds = TextUtilities.getTextBounds(
-                msg, g2, g2.getFontMetrics()
-            );
+            Rectangle2D bounds = TextUtilities.getTextBounds(msg, g2, 
+                    g2.getFontMetrics());
             y += bounds.getHeight();
             g2.drawString(msg, (int) x, (int) y);
         }
@@ -719,9 +762,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
      */
     protected AxisState drawAxis(Graphics2D g2, Rectangle2D plotArea, 
                                  Rectangle2D dataArea) {
-        return this.axis.draw(
-            g2, dataArea.getMinY(), plotArea, dataArea, RectangleEdge.TOP, null
-        );
+        return this.axis.draw(g2, dataArea.getMinY(), plotArea, dataArea, 
+                RectangleEdge.TOP, null);
     }
    
     /**
@@ -742,9 +784,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         if (!DatasetUtilities.isEmptyOrNull(this.dataset)) {
             int seriesCount = this.dataset.getSeriesCount();
             for (int series = 0; series < seriesCount; series++) {
-                this.renderer.drawSeries(
-                    g2, dataArea, info, this, this.dataset, series
-                );
+                this.renderer.drawSeries(g2, dataArea, info, this, 
+                        this.dataset, series);
             }
         }
         else {
@@ -773,9 +814,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
             Stroke gridStroke = getAngleGridlineStroke();
             Paint gridPaint = getAngleGridlinePaint();
             if ((gridStroke != null) && (gridPaint != null)) {
-                this.renderer.drawAngularGridLines(
-                    g2, this, angularTicks, dataArea
-                );
+                this.renderer.drawAngularGridLines(g2, this, angularTicks, 
+                        dataArea);
             }
         }
        
@@ -784,9 +824,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
             Stroke gridStroke = getRadiusGridlineStroke();
             Paint gridPaint = getRadiusGridlinePaint();
             if ((gridStroke != null) && (gridPaint != null)) {
-                this.renderer.drawRadialGridLines(
-                    g2, this, this.axis, radialTicks, dataArea
-                );
+                this.renderer.drawRadialGridLines(g2, this, this.axis, 
+                        radialTicks, dataArea);
             }
         }      
     }
@@ -908,9 +947,6 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         if (!(obj instanceof PolarPlot)) {
             return false;
         }
-        if (!super.equals(obj)) {
-            return false;
-        }
         PolarPlot that = (PolarPlot) obj;
         if (!ObjectUtilities.equal(this.axis, that.axis)) {
             return false;
@@ -930,9 +966,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         if (!PaintUtilities.equal(this.angleLabelPaint, that.angleLabelPaint)) {
             return false;   
         }
-        if (!ObjectUtilities.equal(
-            this.angleGridlineStroke, that.angleGridlineStroke
-        )) {
+        if (!ObjectUtilities.equal(this.angleGridlineStroke, 
+                that.angleGridlineStroke)) {
             return false;
         }
         if (!PaintUtilities.equal(
@@ -943,17 +978,18 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         if (this.radiusGridlinesVisible != that.radiusGridlinesVisible) {
             return false;
         }
-        if (!ObjectUtilities.equal(
-            this.radiusGridlineStroke, that.radiusGridlineStroke
-        )) {
+        if (!ObjectUtilities.equal(this.radiusGridlineStroke, 
+                that.radiusGridlineStroke)) {
             return false;
         }
-        if (!PaintUtilities.equal(
-            this.radiusGridlinePaint, that.radiusGridlinePaint
-        )) {
+        if (!PaintUtilities.equal(this.radiusGridlinePaint, 
+                that.radiusGridlinePaint)) {
             return false;
         }
-        return true;
+        if (!this.cornerTextItems.equals(that.cornerTextItems)) {
+            return false;
+        }
+        return super.equals(obj);
     }
    
     /**
@@ -981,6 +1017,8 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
             clone.renderer 
                 = (PolarItemRenderer) ObjectUtilities.clone(this.renderer);
         }
+        
+        clone.cornerTextItems = new ArrayList(this.cornerTextItems);
        
         return clone;
     }
@@ -998,6 +1036,7 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         SerialUtilities.writePaint(this.angleGridlinePaint, stream);
         SerialUtilities.writeStroke(this.radiusGridlineStroke, stream);
         SerialUtilities.writePaint(this.radiusGridlinePaint, stream);
+        SerialUtilities.writePaint(this.angleLabelPaint, stream);
     }
    
     /**
@@ -1016,6 +1055,7 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         this.angleGridlinePaint = SerialUtilities.readPaint(stream);
         this.radiusGridlineStroke = SerialUtilities.readStroke(stream);
         this.radiusGridlinePaint = SerialUtilities.readPaint(stream);
+        this.angleLabelPaint = SerialUtilities.readPaint(stream);
       
         if (this.axis != null) {
             this.axis.setPlot(this);
@@ -1040,6 +1080,22 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         // do nothing
     }
    
+    /**
+     * This method is required by the {@link Zoomable} interface, but since
+     * the plot does not have any domain axes, it does nothing.
+     *
+     * @param factor  the zoom factor.
+     * @param state  the plot state.
+     * @param source  the source point (in Java2D coordinates).
+     * @param useAnchor  use source point as zoom anchor?
+     * 
+     * @since 1.0.7
+     */
+    public void zoomDomainAxes(double factor, PlotRenderingInfo state, 
+                               Point2D source, boolean useAnchor) {
+        // do nothing
+    }
+   
     /**
      * This method is required by the {@link Zoomable} interface, but since
      * the plot does not have any domain axes, it does nothing.
@@ -1053,7 +1109,7 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
                                PlotRenderingInfo state, Point2D source) {
         // do nothing
     }
-   
+
     /**
      * Multiplies the range on the range axis/axes by the specified factor.
      *
@@ -1066,6 +1122,35 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         zoom(factor);
     }
    
+    /**
+     * Multiplies the range on the range axis by the specified factor.
+     *
+     * @param factor  the zoom factor.
+     * @param info  the plot rendering info.
+     * @param source  the source point (in Java2D space).
+     * @param useAnchor  use source point as zoom anchor?
+     * 
+     * @see #zoomDomainAxes(double, PlotRenderingInfo, Point2D, boolean)
+     * 
+     * @since 1.0.7
+     */
+    public void zoomRangeAxes(double factor, PlotRenderingInfo info,
+                              Point2D source, boolean useAnchor) {
+                
+        if (useAnchor) {
+            // get the source coordinate - this plot has always a VERTICAL
+            // orientation
+            double sourceX = source.getX();
+            double anchorX = this.axis.java2DToValue(sourceX, 
+                    info.getDataArea(), RectangleEdge.BOTTOM);
+            this.axis.resizeRange(factor, anchorX);
+        }
+        else {
+            this.axis.resizeRange(factor);
+        }
+        
+    }
+    
     /**
      * Zooms in on the range axes.
      * 
@@ -1080,18 +1165,18 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     }   
 
     /**
-     * Returns <code>true</code>.
+     * Returns <code>false</code> always.
      * 
-     * @return A boolean.
+     * @return <code>false</code> always.
      */
     public boolean isDomainZoomable() {
         return false;
     }
     
     /**
-     * Returns <code>true</code>.
+     * Returns <code>true</code> to indicate that the range axis is zoomable.
      * 
-     * @return A boolean.
+     * @return <code>true</code>.
      */
     public boolean isRangeZoomable() {
         return true;
@@ -1106,11 +1191,6 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
         return PlotOrientation.HORIZONTAL;
     }
 
-
-    // ----------------------
-    // --- Public Methods ---
-    // ----------------------
-
     /**
      * Returns the upper bound of the radius axis.
      * 
@@ -1121,7 +1201,9 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
     }
 
     /**
-     * Translates a (theta, radius) pair into Java2D coordinates.
+     * Translates a (theta, radius) pair into Java2D coordinates.  If 
+     * <code>radius</code> is less than the lower bound of the axis, then
+     * this method returns the centre point.
      * 
      * @param angleDegrees  the angle in degrees.
      * @param radius  the radius.
@@ -1149,13 +1231,14 @@ public class PolarPlot extends Plot implements ValueAxisPlot,
       
         double axisMin = this.axis.getLowerBound();
         double axisMax =  getMaxRadius();
+        double adjustedRadius = Math.max(radius, axisMin);
 
         double xv = length / 2.0 * Math.cos(radians);
         double yv = length / 2.0 * Math.sin(radians);
 
-        float x = (float) (midX + (xv * (radius - axisMin) 
+        float x = (float) (midX + (xv * (adjustedRadius - axisMin) 
                 / (axisMax - axisMin)));
-        float y = (float) (midY + (yv * (radius - axisMin) 
+        float y = (float) (midY + (yv * (adjustedRadius - axisMin) 
                 / (axisMax - axisMin)));
       
         int ix = Math.round(x);
