@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,15 +27,13 @@
  * --------------------------
  * BoxAndWhiskerRenderer.java
  * --------------------------
- * (C) Copyright 2003-2005, by David Browning and Contributors.
+ * (C) Copyright 2003-2008, by David Browning and Contributors.
  *
  * Original Author:  David Browning (for the Australian Institute of Marine 
  *                   Science);
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *                   Tim Bardzil;
  *
- * $Id: BoxAndWhiskerRenderer.java,v 1.8.2.5 2005/12/01 17:21:14 mungady Exp $
- *
  * Changes
  * -------
  * 21-Aug-2003 : Version 1, contributed by David Browning (for the Australian 
@@ -57,7 +55,16 @@
  *               are shown as blocks (DG);
  * 20-Apr-2005 : Generate legend labels, tooltips and URLs (DG);
  * 09-Jun-2005 : Updated equals() to handle GradientPaint (DG);
- * 
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 12-Oct-2006 : Source reformatting and API doc updates (DG);
+ * 12-Oct-2006 : Fixed bug 1572478, potential NullPointerException (DG);
+ * 05-Feb-2006 : Added event notifications to a couple of methods (DG);
+ * 20-Apr-2007 : Updated getLegendItem() for renderer change (DG);
+ * 11-May-2007 : Added check for visibility in getLegendItem() (DG);
+ * 17-May-2007 : Set datasetIndex and seriesIndex in getLegendItem() (DG);
+ * 18-May-2007 : Set dataset and seriesKey for LegendItem (DG);
+ * 03-Jan-2008 : Check visibility of average marker before drawing it (DG);
+ *
  */
 
 package org.jfree.chart.renderer.category;
@@ -101,7 +108,9 @@ import org.jfree.util.PaintUtilities;
 import org.jfree.util.PublicCloneable;
 
 /**
- * A box-and-whisker renderer.
+ * A box-and-whisker renderer.  This renderer requires a 
+ * {@link BoxAndWhiskerCategoryDataset} and is for use with the 
+ * {@link CategoryPlot} class.
  */
 public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer 
                                    implements Cloneable, PublicCloneable, 
@@ -131,25 +140,37 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     /**
      * Returns the paint used to color the median and average markers.
      * 
-     * @return A paint.
+     * @return The paint used to draw the median and average markers (never
+     *     <code>null</code>).
+     *
+     * @see #setArtifactPaint(Paint)
      */
     public Paint getArtifactPaint() {
         return this.artifactPaint;
     }
 
     /**
-     * Sets the paint used to color the median and average markers.
+     * Sets the paint used to color the median and average markers and sends
+     * a {@link RendererChangeEvent} to all registered listeners.
      * 
-     * @param paint  the paint.
+     * @param paint  the paint (<code>null</code> not permitted).
+     *
+     * @see #getArtifactPaint()
      */
     public void setArtifactPaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.artifactPaint = paint;
+        fireChangeEvent();
     }
 
     /**
      * Returns the flag that controls whether or not the box is filled.
      * 
      * @return A boolean.
+     *
+     * @see #setFillBox(boolean)
      */
     public boolean getFillBox() {
         return this.fillBox;   
@@ -160,10 +181,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param flag  the flag.
+     *
+     * @see #getFillBox()
      */
     public void setFillBox(boolean flag) {
         this.fillBox = flag;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -171,18 +194,24 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * that is allocated to the space between items in the chart.
      * 
      * @return The margin.
+     *
+     * @see #setItemMargin(double)
      */
     public double getItemMargin() {
         return this.itemMargin;
     }
 
     /**
-     * Sets the item margin.
+     * Sets the item margin and sends a {@link RendererChangeEvent} to all
+     * registered listeners.
      * 
-     * @param margin  the margin.
+     * @param margin  the margin (a percentage).
+     *
+     * @see #getItemMargin()
      */
     public void setItemMargin(double margin) {
         this.itemMargin = margin;
+        fireChangeEvent();
     }
 
     /**
@@ -191,7 +220,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * @param datasetIndex  the dataset index (zero-based).
      * @param series  the series index (zero-based).
      *
-     * @return The legend item.
+     * @return The legend item (possibly <code>null</code>).
      */
     public LegendItem getLegendItem(int datasetIndex, int series) {
 
@@ -200,31 +229,36 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
             return null;
         }
 
-        CategoryDataset dataset;
-        dataset = cp.getDataset(datasetIndex);
-        String label = getLegendItemLabelGenerator().generateLabel(
-            dataset, series
-        );
+        // check that a legend item needs to be displayed...
+        if (!isSeriesVisible(series) || !isSeriesVisibleInLegend(series)) {
+            return null;
+        }
+
+        CategoryDataset dataset = cp.getDataset(datasetIndex);
+        String label = getLegendItemLabelGenerator().generateLabel(dataset, 
+                series);
         String description = label;
         String toolTipText = null; 
         if (getLegendItemToolTipGenerator() != null) {
             toolTipText = getLegendItemToolTipGenerator().generateLabel(
-                dataset, series
-            );   
+                    dataset, series);   
         }
         String urlText = null;
         if (getLegendItemURLGenerator() != null) {
-            urlText = getLegendItemURLGenerator().generateLabel(
-                dataset, series
-            );   
+            urlText = getLegendItemURLGenerator().generateLabel(dataset, 
+                    series);   
         }
         Shape shape = new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0);
-        Paint paint = getSeriesPaint(series);
-        Paint outlinePaint = getSeriesOutlinePaint(series);
-        Stroke outlineStroke = getSeriesOutlineStroke(series);
-
-        return new LegendItem(label, description, toolTipText, urlText, 
-            shape, paint, outlineStroke, outlinePaint);
+        Paint paint = lookupSeriesPaint(series);
+        Paint outlinePaint = lookupSeriesOutlinePaint(series);
+        Stroke outlineStroke = lookupSeriesOutlineStroke(series);
+        LegendItem result = new LegendItem(label, description, toolTipText, 
+                urlText, shape, paint, outlineStroke, outlinePaint);
+        result.setDataset(dataset);
+        result.setDatasetIndex(datasetIndex);
+        result.setSeriesKey(dataset.getRowKey(series));
+        result.setSeriesIndex(series);
+        return result;
 
     }
 
@@ -246,9 +280,8 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                                                 int rendererIndex,
                                                 PlotRenderingInfo info) {
 
-        CategoryItemRendererState state = super.initialise(
-            g2, dataArea, plot, rendererIndex, info
-        );
+        CategoryItemRendererState state = super.initialise(g2, dataArea, plot,
+                rendererIndex, info);
 
         // calculate the box width
         CategoryAxis domainAxis = getDomainAxis(plot, rendererIndex);
@@ -276,9 +309,8 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                                      - domainAxis.getUpperMargin()
                                      - categoryMargin - currentItemMargin);
             if ((rows * columns) > 0) {
-                state.setBarWidth(
-                    used / (dataset.getColumnCount() * dataset.getRowCount())
-                );
+                state.setBarWidth(used / (dataset.getColumnCount() 
+                        * dataset.getRowCount()));
             }
             else {
                 state.setBarWidth(used);
@@ -316,24 +348,19 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                              
         if (!(dataset instanceof BoxAndWhiskerCategoryDataset)) {
             throw new IllegalArgumentException(
-                "BoxAndWhiskerRenderer.drawItem() : the data should be of type "
-                + "BoxAndWhiskerCategoryDataset only."
-            );
+                    "BoxAndWhiskerRenderer.drawItem() : the data should be " 
+                    + "of type BoxAndWhiskerCategoryDataset only.");
         }
 
         PlotOrientation orientation = plot.getOrientation();
 
         if (orientation == PlotOrientation.HORIZONTAL) {
-            drawHorizontalItem(
-                g2, state, dataArea, plot, domainAxis, rangeAxis, 
-                dataset, row, column
-            );
+            drawHorizontalItem(g2, state, dataArea, plot, domainAxis, 
+                    rangeAxis, dataset, row, column);
         } 
         else if (orientation == PlotOrientation.VERTICAL) {
-            drawVerticalItem(
-                g2, state, dataArea, plot, domainAxis, rangeAxis, 
-                dataset, row, column
-            );
+            drawVerticalItem(g2, state, dataArea, plot, domainAxis, 
+                    rangeAxis, dataset, row, column);
         }
         
     }
@@ -364,14 +391,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                                    int column) {
 
         BoxAndWhiskerCategoryDataset bawDataset 
-            = (BoxAndWhiskerCategoryDataset) dataset;
+                = (BoxAndWhiskerCategoryDataset) dataset;
 
-        double categoryEnd = domainAxis.getCategoryEnd(
-            column, getColumnCount(), dataArea, plot.getDomainAxisEdge()
-        );
-        double categoryStart = domainAxis.getCategoryStart(
-            column, getColumnCount(), dataArea, plot.getDomainAxisEdge()
-        );
+        double categoryEnd = domainAxis.getCategoryEnd(column, 
+                getColumnCount(), dataArea, plot.getDomainAxisEdge());
+        double categoryStart = domainAxis.getCategoryStart(column, 
+                getColumnCount(), dataArea, plot.getDomainAxisEdge());
         double categoryWidth = Math.abs(categoryEnd - categoryStart);
 
         double yy = categoryStart;
@@ -412,37 +437,29 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         Shape box = null;
         if (xQ1 != null && xQ3 != null && xMax != null && xMin != null) {
 
-            double xxQ1 = rangeAxis.valueToJava2D(
-                xQ1.doubleValue(), dataArea, location
-            );
-            double xxQ3 = rangeAxis.valueToJava2D(
-                xQ3.doubleValue(), dataArea, location
-            );
-            double xxMax = rangeAxis.valueToJava2D(
-                xMax.doubleValue(), dataArea, location
-            );
-            double xxMin = rangeAxis.valueToJava2D(
-                xMin.doubleValue(), dataArea, location
-            );
+            double xxQ1 = rangeAxis.valueToJava2D(xQ1.doubleValue(), dataArea, 
+                    location);
+            double xxQ3 = rangeAxis.valueToJava2D(xQ3.doubleValue(), dataArea,
+                    location);
+            double xxMax = rangeAxis.valueToJava2D(xMax.doubleValue(), dataArea,
+                    location);
+            double xxMin = rangeAxis.valueToJava2D(xMin.doubleValue(), dataArea,
+                    location);
             double yymid = yy + state.getBarWidth() / 2.0;
             
             // draw the upper shadow...
             g2.draw(new Line2D.Double(xxMax, yymid, xxQ3, yymid));
-            g2.draw(
-                new Line2D.Double(xxMax, yy, xxMax, yy + state.getBarWidth())
-            );
+            g2.draw(new Line2D.Double(xxMax, yy, xxMax, 
+                    yy + state.getBarWidth()));
 
             // draw the lower shadow...
             g2.draw(new Line2D.Double(xxMin, yymid, xxQ1, yymid));
-            g2.draw(
-                new Line2D.Double(xxMin, yy, xxMin, yy + state.getBarWidth())
-            );
+            g2.draw(new Line2D.Double(xxMin, yy, xxMin,
+                    yy + state.getBarWidth()));
 
             // draw the box...
-            box = new Rectangle2D.Double(
-                Math.min(xxQ1, xxQ3), yy, Math.abs(xxQ1 - xxQ3), 
-                state.getBarWidth()
-            );
+            box = new Rectangle2D.Double(Math.min(xxQ1, xxQ3), yy, 
+                    Math.abs(xxQ1 - xxQ3), state.getBarWidth());
             if (this.fillBox) {
                 g2.fill(box);
             } 
@@ -456,50 +473,47 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         // draw mean - SPECIAL AIMS REQUIREMENT...
         Number xMean = bawDataset.getMeanValue(row, column);
         if (xMean != null) {
-            double xxMean = rangeAxis.valueToJava2D(
-                xMean.doubleValue(), dataArea, location
-            );
+            double xxMean = rangeAxis.valueToJava2D(xMean.doubleValue(), 
+                    dataArea, location);
             aRadius = state.getBarWidth() / 4;
-            Ellipse2D.Double avgEllipse = new Ellipse2D.Double(
-                xxMean - aRadius, yy + aRadius, aRadius * 2, aRadius * 2
-            );
-            g2.fill(avgEllipse);
-            g2.draw(avgEllipse);
+            // here we check that the average marker will in fact be visible
+            // before drawing it...
+            if ((xxMean > (dataArea.getMinX() - aRadius)) 
+                    && (xxMean < (dataArea.getMaxX() + aRadius))) {
+                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xxMean 
+                        - aRadius, yy + aRadius, aRadius * 2, aRadius * 2);
+                g2.fill(avgEllipse);
+                g2.draw(avgEllipse);
+            }
         }
 
         // draw median...
         Number xMedian = bawDataset.getMedianValue(row, column);
         if (xMedian != null) {
-            double xxMedian = rangeAxis.valueToJava2D(
-                xMedian.doubleValue(), dataArea, location
-            );
-            g2.draw(
-                new Line2D.Double(
-                    xxMedian, yy, xxMedian, yy + state.getBarWidth()
-                )
-            );
+            double xxMedian = rangeAxis.valueToJava2D(xMedian.doubleValue(), 
+                    dataArea, location);
+            g2.draw(new Line2D.Double(xxMedian, yy, xxMedian, 
+                    yy + state.getBarWidth()));
         }
         
         // collect entity and tool tip information...
-        if (state.getInfo() != null) {
+        if (state.getInfo() != null && box != null) {
             EntityCollection entities = state.getEntityCollection();
             if (entities != null) {
                 String tip = null;
                 CategoryToolTipGenerator tipster 
-                    = getToolTipGenerator(row, column);
+                        = getToolTipGenerator(row, column);
                 if (tipster != null) {
                     tip = tipster.generateToolTip(dataset, row, column);
                 }
                 String url = null;
                 if (getItemURLGenerator(row, column) != null) {
                     url = getItemURLGenerator(row, column).generateURL(
-                        dataset, row, column
-                    );
+                            dataset, row, column);
                 }
-                CategoryItemEntity entity = new CategoryItemEntity(
-                    box, tip, url, dataset, row, dataset.getColumnKey(column), 
-                    column
-                );
+                CategoryItemEntity entity = new CategoryItemEntity(box, tip, 
+                        url, dataset, dataset.getRowKey(row), 
+                        dataset.getColumnKey(column));
                 entities.add(entity);
             }
         }
@@ -532,14 +546,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                                  int column) {
 
         BoxAndWhiskerCategoryDataset bawDataset 
-            = (BoxAndWhiskerCategoryDataset) dataset;
+                = (BoxAndWhiskerCategoryDataset) dataset;
         
-        double categoryEnd = domainAxis.getCategoryEnd(
-            column, getColumnCount(), dataArea, plot.getDomainAxisEdge()
-        );
-        double categoryStart = domainAxis.getCategoryStart(
-            column, getColumnCount(), dataArea, plot.getDomainAxisEdge()
-        );
+        double categoryEnd = domainAxis.getCategoryEnd(column, 
+                getColumnCount(), dataArea, plot.getDomainAxisEdge());
+        double categoryStart = domainAxis.getCategoryStart(column, 
+                getColumnCount(), dataArea, plot.getDomainAxisEdge());
         double categoryWidth = categoryEnd - categoryStart;
 
         double xx = categoryStart;
@@ -584,37 +596,29 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         Shape box = null;
         if (yQ1 != null && yQ3 != null && yMax != null && yMin != null) {
 
-            double yyQ1 = rangeAxis.valueToJava2D(
-                yQ1.doubleValue(), dataArea, location
-            );
-            double yyQ3 = rangeAxis.valueToJava2D(
-                yQ3.doubleValue(), dataArea, location
-            );
-            double yyMax = rangeAxis.valueToJava2D(
-                yMax.doubleValue(), dataArea, location
-            );
-            double yyMin = rangeAxis.valueToJava2D(
-                yMin.doubleValue(), dataArea, location
-            );
+            double yyQ1 = rangeAxis.valueToJava2D(yQ1.doubleValue(), dataArea,
+                    location);
+            double yyQ3 = rangeAxis.valueToJava2D(yQ3.doubleValue(), dataArea, 
+                    location);
+            double yyMax = rangeAxis.valueToJava2D(yMax.doubleValue(), 
+                    dataArea, location);
+            double yyMin = rangeAxis.valueToJava2D(yMin.doubleValue(), 
+                    dataArea, location);
             double xxmid = xx + state.getBarWidth() / 2.0;
             
             // draw the upper shadow...
             g2.draw(new Line2D.Double(xxmid, yyMax, xxmid, yyQ3));
-            g2.draw(
-                new Line2D.Double(xx, yyMax, xx + state.getBarWidth(), yyMax)
-            );
+            g2.draw(new Line2D.Double(xx, yyMax, xx + state.getBarWidth(), 
+                    yyMax));
 
             // draw the lower shadow...
             g2.draw(new Line2D.Double(xxmid, yyMin, xxmid, yyQ1));
-            g2.draw(
-                new Line2D.Double(xx, yyMin, xx + state.getBarWidth(), yyMin)
-            );
+            g2.draw(new Line2D.Double(xx, yyMin, xx + state.getBarWidth(), 
+                    yyMin));
 
             // draw the body...
-            box = new Rectangle2D.Double(
-                xx, Math.min(yyQ1, yyQ3), state.getBarWidth(), 
-                Math.abs(yyQ1 - yyQ3)
-            );
+            box = new Rectangle2D.Double(xx, Math.min(yyQ1, yyQ3), 
+                    state.getBarWidth(), Math.abs(yyQ1 - yyQ3));
             if (this.fillBox) {
                 g2.fill(box);
             }
@@ -627,37 +631,34 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         // draw mean - SPECIAL AIMS REQUIREMENT...
         Number yMean = bawDataset.getMeanValue(row, column);
         if (yMean != null) {
-            yyAverage = rangeAxis.valueToJava2D(
-                yMean.doubleValue(), dataArea, location
-            );
+            yyAverage = rangeAxis.valueToJava2D(yMean.doubleValue(), 
+                    dataArea, location);
             aRadius = state.getBarWidth() / 4;
-            Ellipse2D.Double avgEllipse = new Ellipse2D.Double(
-                xx + aRadius, yyAverage - aRadius, aRadius * 2, aRadius * 2
-            );
-            g2.fill(avgEllipse);
-            g2.draw(avgEllipse);
+            // here we check that the average marker will in fact be visible
+            // before drawing it...
+            if ((yyAverage > (dataArea.getMinY() - aRadius)) 
+                    && (yyAverage < (dataArea.getMaxY() + aRadius))) {
+                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xx + aRadius, 
+                        yyAverage - aRadius, aRadius * 2, aRadius * 2);
+                g2.fill(avgEllipse);
+                g2.draw(avgEllipse);
+            }
         }
 
         // draw median...
         Number yMedian = bawDataset.getMedianValue(row, column);
         if (yMedian != null) {
-            double yyMedian = rangeAxis.valueToJava2D(
-                yMedian.doubleValue(), dataArea, location
-            );
-            g2.draw(
-                new Line2D.Double(
-                    xx, yyMedian, xx + state.getBarWidth(), yyMedian
-                )
-            );
+            double yyMedian = rangeAxis.valueToJava2D(yMedian.doubleValue(), 
+                    dataArea, location);
+            g2.draw(new Line2D.Double(xx, yyMedian, xx + state.getBarWidth(), 
+                    yyMedian));
         }
         
         // draw yOutliers...
         double maxAxisValue = rangeAxis.valueToJava2D(
-            rangeAxis.getUpperBound(), dataArea, location
-        ) + aRadius;
+                rangeAxis.getUpperBound(), dataArea, location) + aRadius;
         double minAxisValue = rangeAxis.valueToJava2D(
-            rangeAxis.getLowerBound(), dataArea, location
-        ) - aRadius;
+                rangeAxis.getLowerBound(), dataArea, location) - aRadius;
 
         g2.setPaint(p);
 
@@ -665,7 +666,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         double oRadius = state.getBarWidth() / 3;    // outlier radius
         List outliers = new ArrayList();
         OutlierListCollection outlierListCollection 
-            = new OutlierListCollection();
+                = new OutlierListCollection();
 
         // From outlier array sort out which are outliers and put these into a 
         // list If there are any farouts, set the flag on the 
@@ -685,24 +686,16 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                     outlierListCollection.setLowFarOut(true);
                 }
                 else if (outlier > maxRegular.doubleValue()) {
-                    yyOutlier = rangeAxis.valueToJava2D(
-                        outlier, dataArea, location
-                    );
-                    outliers.add(
-                        new Outlier(
-                            xx + state.getBarWidth() / 2.0, yyOutlier, oRadius
-                        )
-                    );
+                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea, 
+                            location);
+                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0, 
+                            yyOutlier, oRadius));
                 }
                 else if (outlier < minRegular.doubleValue()) {
-                    yyOutlier = rangeAxis.valueToJava2D(
-                        outlier, dataArea, location
-                    );
-                    outliers.add(
-                        new Outlier(
-                            xx + state.getBarWidth() / 2.0, yyOutlier, oRadius
-                        )
-                    );
+                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea, 
+                            location);
+                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0, 
+                            yyOutlier, oRadius));
                 }
                 Collections.sort(outliers);
             }
@@ -715,15 +708,14 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
             }
 
             for (Iterator iterator = outlierListCollection.iterator(); 
-                 iterator.hasNext();) {
+                     iterator.hasNext();) {
                 OutlierList list = (OutlierList) iterator.next();
                 Outlier outlier = list.getAveragedOutlier();
                 Point2D point = outlier.getPoint();
 
                 if (list.isMultiple()) {
-                    drawMultipleEllipse(
-                        point, state.getBarWidth(), oRadius, g2
-                    );
+                    drawMultipleEllipse(point, state.getBarWidth(), oRadius, 
+                            g2);
                 } 
                 else {
                     drawEllipse(point, oRadius, g2);
@@ -732,39 +724,33 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
             // draw farout indicators
             if (outlierListCollection.isHighFarOut()) {
-                drawHighFarOut(
-                    aRadius / 2.0, g2, xx + state.getBarWidth() / 2.0, 
-                    maxAxisValue
-                );
+                drawHighFarOut(aRadius / 2.0, g2, 
+                        xx + state.getBarWidth() / 2.0, maxAxisValue);
             }
         
             if (outlierListCollection.isLowFarOut()) {
-                drawLowFarOut(
-                    aRadius / 2.0, g2, xx + state.getBarWidth() / 2.0, 
-                    minAxisValue
-                );
+                drawLowFarOut(aRadius / 2.0, g2, 
+                        xx + state.getBarWidth() / 2.0, minAxisValue);
             }
         }
         // collect entity and tool tip information...
-        if (state.getInfo() != null) {
+        if (state.getInfo() != null && box != null) {
             EntityCollection entities = state.getEntityCollection();
             if (entities != null) {
                 String tip = null;
                 CategoryToolTipGenerator tipster 
-                    = getToolTipGenerator(row, column);
+                        = getToolTipGenerator(row, column);
                 if (tipster != null) {
                     tip = tipster.generateToolTip(dataset, row, column);
                 }
                 String url = null;
                 if (getItemURLGenerator(row, column) != null) {
-                    url = getItemURLGenerator(row, column).generateURL(
-                        dataset, row, column
-                    );
+                    url = getItemURLGenerator(row, column).generateURL(dataset,
+                            row, column);
                 }
-                CategoryItemEntity entity = new CategoryItemEntity(
-                    box, tip, url, dataset, row, dataset.getColumnKey(column), 
-                    column
-                );
+                CategoryItemEntity entity = new CategoryItemEntity(box, tip, 
+                        url, dataset, dataset.getRowKey(row), 
+                        dataset.getColumnKey(column));
                 entities.add(entity);
             }
         }
@@ -779,9 +765,8 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * @param g2  the graphics device.
      */
     private void drawEllipse(Point2D point, double oRadius, Graphics2D g2) {
-        Ellipse2D dot = new Ellipse2D.Double(
-            point.getX() + oRadius / 2, point.getY(), oRadius, oRadius
-        );
+        Ellipse2D dot = new Ellipse2D.Double(point.getX() + oRadius / 2, 
+                point.getY(), oRadius, oRadius);
         g2.draw(dot);
     }
 
@@ -796,13 +781,10 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     private void drawMultipleEllipse(Point2D point, double boxWidth, 
                                      double oRadius, Graphics2D g2)  {
                                          
-        Ellipse2D dot1 = new Ellipse2D.Double(
-            point.getX() - (boxWidth / 2) + oRadius, point.getY(), 
-            oRadius, oRadius
-        );
-        Ellipse2D dot2 = new Ellipse2D.Double(
-            point.getX() + (boxWidth / 2), point.getY(), oRadius, oRadius
-        );
+        Ellipse2D dot1 = new Ellipse2D.Double(point.getX() - (boxWidth / 2) 
+                + oRadius, point.getY(), oRadius, oRadius);
+        Ellipse2D dot2 = new Ellipse2D.Double(point.getX() + (boxWidth / 2), 
+                point.getY(), oRadius, oRadius);
         g2.draw(dot1);
         g2.draw(dot2);
     }
@@ -890,7 +872,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * @throws ClassNotFoundException  if there is a classpath problem.
      */
     private void readObject(ObjectInputStream stream) 
-        throws IOException, ClassNotFoundException {
+            throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.artifactPaint = SerialUtilities.readPaint(stream);
     }
