@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * -------------------------
  * LineAndShapeRenderer.java
  * -------------------------
- * (C) Copyright 2001-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2001-2007, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Mark Watson (www.markwatson.com);
@@ -35,8 +35,6 @@
  *                   Richard Atkinson;
  *                   Christian W. Zuckschwerdt;
  *
- * $Id: LineAndShapeRenderer.java,v 1.18.2.6 2006/05/25 14:58:06 mungady Exp $
- *
  * Changes
  * -------
  * 23-Oct-2001 : Version 1 (DG);
@@ -79,10 +77,15 @@
  * 13-Apr-2005 : Check flags that control series visibility (DG);
  * 20-Apr-2005 : Use generators for legend labels, tooltips and URLs (DG);
  * 09-Jun-2005 : Use addItemEntity() method (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 25-May-2006 : Added check to drawItem() to detect when both the line and
  *               the shape are not visible (DG);
- * 
+ * 20-Apr-2007 : Updated getLegendItem() for renderer change (DG);
+ * 17-May-2007 : Set datasetIndex and seriesIndex in getLegendItem() (DG);
+ * 18-May-2007 : Set dataset and seriesKey for LegendItem (DG);
+ * 24-Sep-2007 : Deprecated redundant fields/methods (DG);
+ * 27-Sep-2007 : Added option to offset series x-position within category (DG);
+ *
  */
 
 package org.jfree.chart.renderer.category;
@@ -120,7 +123,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
     /** For serialization. */
     private static final long serialVersionUID = -197749519869226398L;
     
-    /** A flag that controls whether or not lines are visible for ALL series. */
+    /** 
+     * A flag that controls whether or not lines are visible for ALL series. 
+     * 
+     * @deprecated As of 1.0.7 (this override flag is unnecessary).
+     */
     private Boolean linesVisible;
 
     /** 
@@ -136,7 +143,9 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
     private boolean baseLinesVisible;
 
     /** 
-     * A flag that controls whether or not shapes are visible for ALL series. 
+     * A flag that controls whether or not shapes are visible for ALL series.
+     * 
+     * @deprecated As of 1.0.7 (this override flag is unnecessary).
      */
     private Boolean shapesVisible;
 
@@ -149,7 +158,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
     /** The default value returned by the getShapeVisible() method. */
     private boolean baseShapesVisible;
 
-    /** A flag that controls whether or not shapes are filled for ALL series. */
+    /** 
+     * A flag that controls whether or not shapes are filled for ALL series. 
+     * 
+     * @deprecated As of 1.0.7 (this override flag is unnecessary).
+     */
     private Boolean shapesFilled;
     
     /** 
@@ -175,7 +188,23 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * outlines - if not, the regular series paint is used. 
      */
     private boolean useOutlinePaint;
+    
+    /**
+     * A flag that controls whether or not the x-position for each item is
+     * offset within the category according to the series.
+     * 
+     * @since 1.0.7
+     */
+    private boolean useSeriesOffset;
 
+    /**
+     * The item margin used for series offsetting - this allows the positioning
+     * to match the bar positions of the {@link BarRenderer} class.
+     * 
+     * @since 1.0.7
+     */
+    private double itemMargin;
+    
     /**
      * Creates a renderer with both lines and shapes visible by default.
      */
@@ -203,6 +232,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         this.useFillPaint = false;
         this.drawOutlines = true;
         this.useOutlinePaint = false;
+        this.useSeriesOffset = false;  // preserves old behaviour
+        this.itemMargin = 0.0;
     }
     
     // LINES VISIBLE
@@ -235,6 +266,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * settings will apply.
      * 
      * @return A flag (possibly <code>null</code>).
+     * 
+     * @see #setLinesVisible(Boolean)
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public Boolean getLinesVisible() {
         return this.linesVisible;   
@@ -247,10 +283,15 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * want the "per series" settings to apply.
      *
      * @param visible  the flag (<code>null</code> permitted).
+     * 
+     * @see #getLinesVisible()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setLinesVisible(Boolean visible) {
         this.linesVisible = visible;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -259,6 +300,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * registered listeners.
      *
      * @param visible  the flag.
+     * 
+     * @see #getLinesVisible()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setLinesVisible(boolean visible) {
         setLinesVisible(BooleanUtilities.valueOf(visible));
@@ -271,27 +317,35 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * @param series  the series index (zero-based).
      *
      * @return The flag (possibly <code>null</code>).
+     * 
+     * @see #setSeriesLinesVisible(int, Boolean)
      */
     public Boolean getSeriesLinesVisible(int series) {
         return this.seriesLinesVisible.getBoolean(series);
     }
 
     /**
-     * Sets the 'lines visible' flag for a series.
+     * Sets the 'lines visible' flag for a series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param series  the series index (zero-based).
      * @param flag  the flag (<code>null</code> permitted).
+     * 
+     * @see #getSeriesLinesVisible(int)
      */
     public void setSeriesLinesVisible(int series, Boolean flag) {
         this.seriesLinesVisible.setBoolean(series, flag);
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
-     * Sets the 'lines visible' flag for a series.
+     * Sets the 'lines visible' flag for a series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param series  the series index (zero-based).
      * @param visible  the flag.
+     * 
+     * @see #getSeriesLinesVisible(int)
      */
     public void setSeriesLinesVisible(int series, boolean visible) {
         setSeriesLinesVisible(series, BooleanUtilities.valueOf(visible));
@@ -301,19 +355,24 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * Returns the base 'lines visible' attribute.
      *
      * @return The base flag.
+     * 
+     * @see #getBaseLinesVisible()
      */
     public boolean getBaseLinesVisible() {
         return this.baseLinesVisible;
     }
 
     /**
-     * Sets the base 'lines visible' flag.
+     * Sets the base 'lines visible' flag and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param flag  the flag.
+     * 
+     * @see #getBaseLinesVisible()
      */
     public void setBaseLinesVisible(boolean flag) {
         this.baseLinesVisible = flag;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     // SHAPES VISIBLE
@@ -345,6 +404,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * items in ALL series.
      * 
      * @return The flag (possibly <code>null</code>).
+     * 
+     * @see #setShapesVisible(Boolean)
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public Boolean getShapesVisible() {
         return this.shapesVisible;    
@@ -355,10 +419,15 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param visible  the flag (<code>null</code> permitted).
+     * 
+     * @see #getShapesVisible()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setShapesVisible(Boolean visible) {
         this.shapesVisible = visible;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -366,6 +435,11 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param visible  the flag.
+     * 
+     * @see #getShapesVisible()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setShapesVisible(boolean visible) {
         setShapesVisible(BooleanUtilities.valueOf(visible));
@@ -378,6 +452,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * @param series  the series index (zero-based).
      *
      * @return A boolean.
+     * 
+     * @see #setSeriesShapesVisible(int, Boolean)
      */
     public Boolean getSeriesShapesVisible(int series) {
         return this.seriesShapesVisible.getBoolean(series);
@@ -389,6 +465,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * 
      * @param series  the series index (zero-based).
      * @param visible  the flag.
+     * 
+     * @see #getSeriesShapesVisible(int)
      */
     public void setSeriesShapesVisible(int series, boolean visible) {
         setSeriesShapesVisible(series, BooleanUtilities.valueOf(visible));
@@ -400,29 +478,36 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      *
      * @param series  the series index (zero-based).
      * @param flag  the flag.
+     * 
+     * @see #getSeriesShapesVisible(int)
      */
     public void setSeriesShapesVisible(int series, Boolean flag) {
         this.seriesShapesVisible.setBoolean(series, flag);
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
      * Returns the base 'shape visible' attribute.
      *
      * @return The base flag.
+     * 
+     * @see #setBaseShapesVisible(boolean)
      */
     public boolean getBaseShapesVisible() {
         return this.baseShapesVisible;
     }
 
     /**
-     * Sets the base 'shapes visible' flag.
+     * Sets the base 'shapes visible' flag and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param flag  the flag.
+     * 
+     * @see #getBaseShapesVisible()
      */
     public void setBaseShapesVisible(boolean flag) {
         this.baseShapesVisible = flag;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -430,6 +515,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * <code>false</code> otherwise.
      * 
      * @return A boolean.
+     * 
+     * @see #setDrawOutlines(boolean)
      */
     public boolean getDrawOutlines() {
         return this.drawOutlines;
@@ -444,10 +531,12 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * this flag allows you to set your own preference.
      * 
      * @param flag  the flag.
+     * 
+     * @see #getDrawOutlines()
      */
     public void setDrawOutlines(boolean flag) {
         this.drawOutlines = flag;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
     
     /**
@@ -455,6 +544,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * shape outlines.  If not, the regular series paint is used.
      * 
      * @return A boolean.
+     * 
+     * @see #setUseOutlinePaint(boolean)
      */
     public boolean getUseOutlinePaint() {
         return this.useOutlinePaint;   
@@ -462,12 +553,16 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
     
     /**
      * Sets the flag that controls whether the outline paint is used for shape 
-     * outlines.
+     * outlines, and sends a {@link RendererChangeEvent} to all registered 
+     * listeners. 
      * 
      * @param use  the flag.
+     * 
+     * @see #getUseOutlinePaint()
      */
     public void setUseOutlinePaint(boolean use) {
         this.useOutlinePaint = use;   
+        fireChangeEvent();
     }
 
     // SHAPES FILLED
@@ -518,15 +613,26 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * ALL series.
      * 
      * @return A Boolean.
+     * 
+     * @see #setShapesFilled(Boolean)
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public Boolean getShapesFilled() {
         return this.shapesFilled;
     }
 
     /**
-     * Sets the 'shapes filled' for ALL series.
+     * Sets the 'shapes filled' for ALL series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param filled  the flag.
+     * 
+     * @see #getShapesFilled()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setShapesFilled(boolean filled) {
         if (filled) {
@@ -538,52 +644,71 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
     }
     
     /**
-     * Sets the 'shapes filled' for ALL series.
+     * Sets the 'shapes filled' for ALL series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param filled  the flag (<code>null</code> permitted).
+     * 
+     * @see #getShapesFilled()
+     * 
+     * @deprecated As of 1.0.7 (the override facility is unnecessary, just
+     *     use the per-series and base (default) settings).
      */
     public void setShapesFilled(Boolean filled) {
         this.shapesFilled = filled;
+        fireChangeEvent();
     }
     
     /**
-     * Sets the 'shapes filled' flag for a series.
+     * Sets the 'shapes filled' flag for a series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param series  the series index (zero-based).
      * @param filled  the flag.
+     * 
+     * @see #getSeriesShapesFilled(int)
      */
     public void setSeriesShapesFilled(int series, Boolean filled) {
         this.seriesShapesFilled.setBoolean(series, filled);
+        fireChangeEvent();
     }
 
     /**
-     * Sets the 'shapes filled' flag for a series.
+     * Sets the 'shapes filled' flag for a series and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param series  the series index (zero-based).
      * @param filled  the flag.
+     * 
+     * @see #getSeriesShapesFilled(int)
      */
     public void setSeriesShapesFilled(int series, boolean filled) {
-        this.seriesShapesFilled.setBoolean(
-            series, BooleanUtilities.valueOf(filled)
-        );
+        // delegate
+        setSeriesShapesFilled(series, BooleanUtilities.valueOf(filled));
     }
 
     /**
      * Returns the base 'shape filled' attribute.
      *
      * @return The base flag.
+     * 
+     * @see #setBaseShapesFilled(boolean)
      */
     public boolean getBaseShapesFilled() {
         return this.baseShapesFilled;
     }
 
     /**
-     * Sets the base 'shapes filled' flag.
+     * Sets the base 'shapes filled' flag and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      *
      * @param flag  the flag.
+     * 
+     * @see #getBaseShapesFilled()
      */
     public void setBaseShapesFilled(boolean flag) {
         this.baseShapesFilled = flag;
+        fireChangeEvent();
     }
 
     /**
@@ -592,6 +717,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * use the regular paint.
      * 
      * @return A boolean.
+     * 
+     * @see #setUseFillPaint(boolean)
      */
     public boolean getUseFillPaint() {
         return this.useFillPaint;
@@ -603,10 +730,79 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
      * registered listeners.
      * 
      * @param flag  the flag.
+     * 
+     * @see #getUseFillPaint()
      */
     public void setUseFillPaint(boolean flag) {
         this.useFillPaint = flag;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
+    }
+    
+    /**
+     * Returns the flag that controls whether or not the x-position for each
+     * data item is offset within the category according to the series.
+     * 
+     * @return A boolean.
+     * 
+     * @see #setUseSeriesOffset(boolean)
+     * 
+     * @since 1.0.7
+     */
+    public boolean getUseSeriesOffset() {
+        return this.useSeriesOffset;
+    }
+    
+    /**
+     * Sets the flag that controls whether or not the x-position for each 
+     * data item is offset within its category according to the series, and
+     * sends a {@link RendererChangeEvent} to all registered listeners.
+     * 
+     * @param offset  the offset.
+     * 
+     * @see #getUseSeriesOffset()
+     * 
+     * @since 1.0.7
+     */
+    public void setUseSeriesOffset(boolean offset) {
+        this.useSeriesOffset = offset;
+        fireChangeEvent();
+    }
+    
+    /**
+     * Returns the item margin, which is the gap between items within a 
+     * category (expressed as a percentage of the overall category width).  
+     * This can be used to match the offset alignment with the bars drawn by 
+     * a {@link BarRenderer}).
+     * 
+     * @return The item margin.
+     * 
+     * @see #setItemMargin(double)
+     * @see #getUseSeriesOffset()
+     * 
+     * @since 1.0.7
+     */
+    public double getItemMargin() {
+        return this.itemMargin;
+    }
+    
+    /**
+     * Sets the item margin, which is the gap between items within a category
+     * (expressed as a percentage of the overall category width), and sends
+     * a {@link RendererChangeEvent} to all registered listeners.
+     * 
+     * @param margin  the margin (0.0 <= margin < 1.0).
+     * 
+     * @see #getItemMargin()
+     * @see #getUseSeriesOffset()
+     * 
+     * @since 1.0.7
+     */
+    public void setItemMargin(double margin) {
+        if (margin < 0.0 || margin >= 1.0) {
+            throw new IllegalArgumentException("Requires 0.0 <= margin < 1.0.");
+        }
+        this.itemMargin = margin;
+        fireChangeEvent();
     }
     
     /**
@@ -625,39 +821,40 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         }
 
         if (isSeriesVisible(series) && isSeriesVisibleInLegend(series)) {
-            CategoryDataset dataset;
-            dataset = cp.getDataset(datasetIndex);
+            CategoryDataset dataset = cp.getDataset(datasetIndex);
             String label = getLegendItemLabelGenerator().generateLabel(
-                dataset, series
-            );
+                    dataset, series);
             String description = label;
             String toolTipText = null; 
             if (getLegendItemToolTipGenerator() != null) {
                 toolTipText = getLegendItemToolTipGenerator().generateLabel(
-                    dataset, series
-                );   
+                        dataset, series);   
             }
             String urlText = null;
             if (getLegendItemURLGenerator() != null) {
                 urlText = getLegendItemURLGenerator().generateLabel(
-                    dataset, series
-                );   
+                        dataset, series);   
             }
-            Shape shape = getSeriesShape(series);
-            Paint paint = getSeriesPaint(series);
+            Shape shape = lookupSeriesShape(series);
+            Paint paint = lookupSeriesPaint(series);
             Paint fillPaint = (this.useFillPaint 
-                ? getItemFillPaint(series, 0) : paint);
+                    ? getItemFillPaint(series, 0) : paint);
             boolean shapeOutlineVisible = this.drawOutlines;
             Paint outlinePaint = (this.useOutlinePaint 
-                ? getItemOutlinePaint(series, 0) : paint);
-            Stroke outlineStroke = getSeriesOutlineStroke(series);
+                    ? getItemOutlinePaint(series, 0) : paint);
+            Stroke outlineStroke = lookupSeriesOutlineStroke(series);
             boolean lineVisible = getItemLineVisible(series, 0);
             boolean shapeVisible = getItemShapeVisible(series, 0);
-            return new LegendItem(label, description, toolTipText, 
+            LegendItem result = new LegendItem(label, description, toolTipText, 
                     urlText, shapeVisible, shape, getItemShapeFilled(series, 0),
                     fillPaint, shapeOutlineVisible, outlinePaint, outlineStroke,
                     lineVisible, new Line2D.Double(-7.0, 0.0, 7.0, 0.0),
                     getItemStroke(series, 0), getItemPaint(series, 0));
+            result.setDataset(dataset);
+            result.setDatasetIndex(datasetIndex);
+            result.setSeriesKey(dataset.getRowKey(series));
+            result.setSeriesIndex(series);
+            return result;
         }
         return null;
 
@@ -711,8 +908,16 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         PlotOrientation orientation = plot.getOrientation();
 
         // current data point...
-        double x1 = domainAxis.getCategoryMiddle(column, getColumnCount(), 
-                dataArea, plot.getDomainAxisEdge());
+        double x1;
+        if (this.useSeriesOffset) {
+            x1 = domainAxis.getCategorySeriesMiddle(dataset.getColumnKey(
+                    column), dataset.getRowKey(row), dataset, this.itemMargin, 
+                    dataArea, plot.getDomainAxisEdge());            
+        }
+        else {
+            x1 = domainAxis.getCategoryMiddle(column, getColumnCount(), 
+                    dataArea, plot.getDomainAxisEdge());
+        }
         double value = v.doubleValue();
         double y1 = rangeAxis.valueToJava2D(value, dataArea, 
                 plot.getRangeAxisEdge());
@@ -723,9 +928,19 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
                 if (previousValue != null) {
                     // previous data point...
                     double previous = previousValue.doubleValue();
-                    double x0 = domainAxis.getCategoryMiddle(column - 1, 
-                            getColumnCount(), dataArea, 
-                            plot.getDomainAxisEdge());
+                    double x0;
+                    if (this.useSeriesOffset) {
+                        x0 = domainAxis.getCategorySeriesMiddle(
+                                dataset.getColumnKey(column - 1), 
+                                dataset.getRowKey(row), dataset, 
+                                this.itemMargin, dataArea, 
+                                plot.getDomainAxisEdge());
+                    }
+                    else {
+                        x0 = domainAxis.getCategoryMiddle(column - 1, 
+                                getColumnCount(), dataArea, 
+                                plot.getDomainAxisEdge());
+                    }
                     double y0 = rangeAxis.valueToJava2D(previous, dataArea, 
                             plot.getRangeAxisEdge());
 
@@ -835,9 +1050,8 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         if (!ObjectUtilities.equal(this.shapesFilled, that.shapesFilled)) {
             return false;
         }
-        if (!ObjectUtilities.equal(
-            this.seriesShapesFilled, that.seriesShapesFilled)
-        ) {
+        if (!ObjectUtilities.equal(this.seriesShapesFilled, 
+                that.seriesShapesFilled)) {
             return false;
         }
         if (this.baseShapesFilled != that.baseShapesFilled) {
@@ -846,10 +1060,13 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         if (this.useOutlinePaint != that.useOutlinePaint) {
             return false;
         }
-        if (!super.equals(obj)) {
+        if (this.useSeriesOffset != that.useSeriesOffset) {
             return false;
         }
-        return true;
+        if (this.itemMargin != that.itemMargin) {
+            return false;
+        }
+        return super.equals(obj);
     }
 
     /**
@@ -864,7 +1081,7 @@ public class LineAndShapeRenderer extends AbstractCategoryItemRenderer
         clone.seriesLinesVisible 
             = (BooleanList) this.seriesLinesVisible.clone();
         clone.seriesShapesVisible 
-            = (BooleanList) this.seriesLinesVisible.clone();
+            = (BooleanList) this.seriesShapesVisible.clone();
         clone.seriesShapesFilled 
             = (BooleanList) this.seriesShapesFilled.clone();
         return clone;
