@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,14 +27,12 @@
  * ---------------------------
  * StatisticalBarRenderer.java
  * ---------------------------
- * (C) Copyright 2002-2006, by Pascal Collet and Contributors.
+ * (C) Copyright 2002-2007, by Pascal Collet and Contributors.
  *
  * Original Author:  Pascal Collet;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *                   Christian W. Zuckschwerdt;
  *
- * $Id: StatisticalBarRenderer.java,v 1.4.2.5 2006/07/12 14:30:11 mungady Exp $
- *
  * Changes
  * -------
  * 21-Aug-2002 : Version 1, contributed by Pascal Collet (DG);
@@ -47,17 +45,24 @@
  * 06-Oct-2003 : Corrected typo in exception message (DG);
  * 05-Nov-2004 : Modified drawItem() signature (DG);
  * 15-Jun-2005 : Added errorIndicatorPaint attribute (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 19-May-2006 : Added support for tooltips and URLs (DG);
  * 12-Jul-2006 : Added support for item labels (DG);
- *
+ * 02-Feb-2007 : Removed author tags all over JFreeChart sources (DG);
+ * 28-Aug-2007 : Fixed NullPointerException - see bug 1779941 (DG);
+ * 14-Nov-2007 : Added errorIndicatorStroke, and fixed bugs with drawBarOutline
+ *               and gradientPaintTransformer attributes being ignored (DG);
+ * 
  */
 
 package org.jfree.chart.renderer.category;
 
+import java.awt.BasicStroke;
 import java.awt.Color;
+import java.awt.GradientPaint;
 import java.awt.Graphics2D;
 import java.awt.Paint;
+import java.awt.Stroke;
 import java.awt.geom.Line2D;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
@@ -75,15 +80,15 @@ import org.jfree.chart.plot.PlotOrientation;
 import org.jfree.data.category.CategoryDataset;
 import org.jfree.data.statistics.StatisticalCategoryDataset;
 import org.jfree.io.SerialUtilities;
+import org.jfree.ui.GradientPaintTransformer;
 import org.jfree.ui.RectangleEdge;
+import org.jfree.util.ObjectUtilities;
 import org.jfree.util.PaintUtilities;
 import org.jfree.util.PublicCloneable;
 
 /**
  * A renderer that handles the drawing a bar plot where
  * each bar has a mean value and a standard deviation line.
- *
- * @author Pascal Collet
  */
 public class StatisticalBarRenderer extends BarRenderer
                                     implements CategoryItemRenderer, 
@@ -96,12 +101,20 @@ public class StatisticalBarRenderer extends BarRenderer
     /** The paint used to show the error indicator. */
     private transient Paint errorIndicatorPaint;
     
+    /**
+     * The stroke used to draw the error indicators. 
+     * 
+     * @since 1.0.8
+     */
+    private transient Stroke errorIndicatorStroke;
+    
     /**
      * Default constructor.
      */
     public StatisticalBarRenderer() {
         super();
         this.errorIndicatorPaint = Color.gray;
+        this.errorIndicatorStroke = new BasicStroke(1.0f);
     }
 
     /**
@@ -109,6 +122,8 @@ public class StatisticalBarRenderer extends BarRenderer
      * 
      * @return The paint used for the error indicators (possibly 
      *         <code>null</code>).
+     *         
+     * @see #setErrorIndicatorPaint(Paint)
      */
     public Paint getErrorIndicatorPaint() {
         return this.errorIndicatorPaint;   
@@ -116,13 +131,47 @@ public class StatisticalBarRenderer extends BarRenderer
 
     /**
      * Sets the paint used for the error indicators (if <code>null</code>, 
-     * the item outline paint is used instead)
+     * the item outline paint is used instead) and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param paint  the paint (<code>null</code> permitted).
+     * 
+     * @see #getErrorIndicatorPaint()
      */
     public void setErrorIndicatorPaint(Paint paint) {
         this.errorIndicatorPaint = paint;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
+    }
+    
+    /**
+     * Returns the stroke used to draw the error indicators.  If this is 
+     * <code>null</code>, the renderer will use the item outline stroke).
+     * 
+     * @return The stroke (possibly <code>null</code>).
+     * 
+     * @see #setErrorIndicatorStroke(Stroke)
+     *
+     * @since 1.0.8
+     */
+    public Stroke getErrorIndicatorStroke() {
+        return this.errorIndicatorStroke;
+    }
+    
+    /**
+     * Sets the stroke used to draw the error indicators, and sends a 
+     * {@link RendererChangeEvent} to all registered listeners.  If you set
+     * this to <code>null</code>, the renderer will use the item outline
+     * stroke.
+     * 
+     * @param stroke  the stroke (<code>null</code> permitted).
+     * 
+     * @see #getErrorIndicatorStroke()
+     * 
+     * @since 1.0.8
+     */
+    public void setErrorIndicatorStroke(Stroke stroke) {
+        this.errorIndicatorStroke = stroke;
+        fireChangeEvent();
     }
     
     /**
@@ -211,7 +260,9 @@ public class StatisticalBarRenderer extends BarRenderer
 
         // BAR X
         Number meanValue = dataset.getMeanValue(row, column);
-
+        if (meanValue == null) {
+            return;
+        }
         double value = meanValue.doubleValue();
         double base = 0.0;
         double lclip = getLowerClip();
@@ -257,44 +308,64 @@ public class StatisticalBarRenderer extends BarRenderer
 
         Rectangle2D bar = new Rectangle2D.Double(rectX, rectY, rectWidth, 
                 rectHeight);
-        Paint seriesPaint = getItemPaint(row, column);
-        g2.setPaint(seriesPaint);
+        Paint itemPaint = getItemPaint(row, column);
+        GradientPaintTransformer t = getGradientPaintTransformer();
+        if (t != null && itemPaint instanceof GradientPaint) {
+            itemPaint = t.transform((GradientPaint) itemPaint, bar);
+        }
+        g2.setPaint(itemPaint);
         g2.fill(bar);
-        if (state.getBarWidth() > 3) {
-            g2.setStroke(getItemStroke(row, column));
-            g2.setPaint(getItemOutlinePaint(row, column));
-            g2.draw(bar);
+        
+        // draw the outline...
+        if (isDrawBarOutline() 
+                && state.getBarWidth() > BAR_OUTLINE_WIDTH_THRESHOLD) {
+            Stroke stroke = getItemOutlineStroke(row, column);
+            Paint paint = getItemOutlinePaint(row, column);
+            if (stroke != null && paint != null) {
+                g2.setStroke(stroke);
+                g2.setPaint(paint);
+                g2.draw(bar);
+            }
         }
 
         // standard deviation lines
-        double valueDelta = dataset.getStdDevValue(row, column).doubleValue();
-        double highVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
-                + valueDelta, dataArea, yAxisLocation);
-        double lowVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
-                - valueDelta, dataArea, yAxisLocation);
+        Number n = dataset.getStdDevValue(row, column);
+        if (n != null) {
+            double valueDelta = n.doubleValue();
+            double highVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
+                    + valueDelta, dataArea, yAxisLocation);
+            double lowVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
+                    - valueDelta, dataArea, yAxisLocation);
 
-        if (this.errorIndicatorPaint != null) {
-            g2.setPaint(this.errorIndicatorPaint);  
+            if (this.errorIndicatorPaint != null) {
+                g2.setPaint(this.errorIndicatorPaint);  
+            }
+            else {
+                g2.setPaint(getItemOutlinePaint(row, column));   
+            }
+            if (this.errorIndicatorStroke != null) {
+                g2.setStroke(this.errorIndicatorStroke);
+            }
+            else {
+                g2.setStroke(getItemOutlineStroke(row, column));
+            }
+            Line2D line = null;
+            line = new Line2D.Double(lowVal, rectY + rectHeight / 2.0d, 
+                                     highVal, rectY + rectHeight / 2.0d);
+            g2.draw(line);
+            line = new Line2D.Double(highVal, rectY + rectHeight * 0.25, 
+                                     highVal, rectY + rectHeight * 0.75);
+            g2.draw(line);
+            line = new Line2D.Double(lowVal, rectY + rectHeight * 0.25, 
+                                     lowVal, rectY + rectHeight * 0.75);
+            g2.draw(line);
         }
-        else {
-            g2.setPaint(getItemOutlinePaint(row, column));   
-        }
-        Line2D line = null;
-        line = new Line2D.Double(lowVal, rectY + rectHeight / 2.0d, 
-                                 highVal, rectY + rectHeight / 2.0d);
-        g2.draw(line);
-        line = new Line2D.Double(highVal, rectY + rectHeight * 0.25, 
-                                 highVal, rectY + rectHeight * 0.75);
-        g2.draw(line);
-        line = new Line2D.Double(lowVal, rectY + rectHeight * 0.25, 
-                                 lowVal, rectY + rectHeight * 0.75);
-        g2.draw(line);
         
         CategoryItemLabelGenerator generator = getItemLabelGenerator(row, 
                 column);
         if (generator != null && isItemLabelVisible(row, column)) {
             drawItemLabel(g2, dataset, row, column, plot, generator, bar, 
-                (value < 0.0));
+                    (value < 0.0));
         }        
 
         // add an item entity, if this information is being collected
@@ -331,9 +402,8 @@ public class StatisticalBarRenderer extends BarRenderer
         RectangleEdge xAxisLocation = plot.getDomainAxisEdge();
         
         // BAR X
-        double rectX = domainAxis.getCategoryStart(
-            column, getColumnCount(), dataArea, xAxisLocation
-        );
+        double rectX = domainAxis.getCategoryStart(column, getColumnCount(), 
+                dataArea, xAxisLocation);
 
         int seriesCount = getRowCount();
         int categoryCount = getColumnCount();
@@ -348,6 +418,9 @@ public class StatisticalBarRenderer extends BarRenderer
 
         // BAR Y
         Number meanValue = dataset.getMeanValue(row, column);
+        if (meanValue == null) {
+            return;
+        }
 
         double value = meanValue.doubleValue();
         double base = 0.0;
@@ -394,44 +467,64 @@ public class StatisticalBarRenderer extends BarRenderer
 
         Rectangle2D bar = new Rectangle2D.Double(rectX, rectY, rectWidth, 
                 rectHeight);
-        Paint seriesPaint = getItemPaint(row, column);
-        g2.setPaint(seriesPaint);
+        Paint itemPaint = getItemPaint(row, column);
+        GradientPaintTransformer t = getGradientPaintTransformer();
+        if (t != null && itemPaint instanceof GradientPaint) {
+            itemPaint = t.transform((GradientPaint) itemPaint, bar);
+        }
+        g2.setPaint(itemPaint);
         g2.fill(bar);
-        if (state.getBarWidth() > 3) {
-            g2.setStroke(getItemStroke(row, column));
-            g2.setPaint(getItemOutlinePaint(row, column));
-            g2.draw(bar);
+        // draw the outline...
+        if (isDrawBarOutline() 
+                && state.getBarWidth() > BAR_OUTLINE_WIDTH_THRESHOLD) {
+            Stroke stroke = getItemOutlineStroke(row, column);
+            Paint paint = getItemOutlinePaint(row, column);
+            if (stroke != null && paint != null) {
+                g2.setStroke(stroke);
+                g2.setPaint(paint);
+                g2.draw(bar);
+            }
         }
 
         // standard deviation lines
-        double valueDelta = dataset.getStdDevValue(row, column).doubleValue();
-        double highVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
-                + valueDelta, dataArea, yAxisLocation);
-        double lowVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
-                - valueDelta, dataArea, yAxisLocation);
+        Number n = dataset.getStdDevValue(row, column);
+        if (n != null) {
+            double valueDelta = n.doubleValue();
+            double highVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
+                    + valueDelta, dataArea, yAxisLocation);
+            double lowVal = rangeAxis.valueToJava2D(meanValue.doubleValue() 
+                    - valueDelta, dataArea, yAxisLocation);
 
-        if (this.errorIndicatorPaint != null) {
-            g2.setPaint(this.errorIndicatorPaint);  
+            if (this.errorIndicatorPaint != null) {
+                g2.setPaint(this.errorIndicatorPaint);  
+            }
+            else {
+                g2.setPaint(getItemOutlinePaint(row, column));   
+            }
+            if (this.errorIndicatorStroke != null) {
+                g2.setStroke(this.errorIndicatorStroke);
+            }
+            else {
+                g2.setStroke(getItemOutlineStroke(row, column));
+            }
+            
+            Line2D line = null;
+            line = new Line2D.Double(rectX + rectWidth / 2.0d, lowVal,
+                                     rectX + rectWidth / 2.0d, highVal);
+            g2.draw(line);
+            line = new Line2D.Double(rectX + rectWidth / 2.0d - 5.0d, highVal,
+                                     rectX + rectWidth / 2.0d + 5.0d, highVal);
+            g2.draw(line);
+            line = new Line2D.Double(rectX + rectWidth / 2.0d - 5.0d, lowVal,
+                                     rectX + rectWidth / 2.0d + 5.0d, lowVal);
+            g2.draw(line);
         }
-        else {
-            g2.setPaint(getItemOutlinePaint(row, column));   
-        }
-        Line2D line = null;
-        line = new Line2D.Double(rectX + rectWidth / 2.0d, lowVal,
-                                 rectX + rectWidth / 2.0d, highVal);
-        g2.draw(line);
-        line = new Line2D.Double(rectX + rectWidth / 2.0d - 5.0d, highVal,
-                                 rectX + rectWidth / 2.0d + 5.0d, highVal);
-        g2.draw(line);
-        line = new Line2D.Double(rectX + rectWidth / 2.0d - 5.0d, lowVal,
-                                 rectX + rectWidth / 2.0d + 5.0d, lowVal);
-        g2.draw(line);
         
         CategoryItemLabelGenerator generator = getItemLabelGenerator(row, 
                 column);
         if (generator != null && isItemLabelVisible(row, column)) {
             drawItemLabel(g2, dataset, row, column, plot, generator, bar, 
-                (value < 0.0));
+                    (value < 0.0));
         }        
 
         // add an item entity, if this information is being collected
@@ -455,15 +548,16 @@ public class StatisticalBarRenderer extends BarRenderer
         if (!(obj instanceof StatisticalBarRenderer)) {
             return false;   
         }
-        if (!super.equals(obj)) {
-            return false;   
-        }
         StatisticalBarRenderer that = (StatisticalBarRenderer) obj;
         if (!PaintUtilities.equal(this.errorIndicatorPaint, 
                 that.errorIndicatorPaint)) {
             return false;
         }
-        return true;
+        if (!ObjectUtilities.equal(this.errorIndicatorStroke, 
+                that.errorIndicatorStroke)) {
+            return false;
+        }
+        return super.equals(obj);
     }
     
     /**
@@ -476,6 +570,7 @@ public class StatisticalBarRenderer extends BarRenderer
     private void writeObject(ObjectOutputStream stream) throws IOException {
         stream.defaultWriteObject();
         SerialUtilities.writePaint(this.errorIndicatorPaint, stream);
+        SerialUtilities.writeStroke(this.errorIndicatorStroke, stream);
     }
 
     /**
@@ -490,6 +585,7 @@ public class StatisticalBarRenderer extends BarRenderer
         throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.errorIndicatorPaint = SerialUtilities.readPaint(stream);
+        this.errorIndicatorStroke = SerialUtilities.readStroke(stream);
     }
 
 }
