@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,15 +27,13 @@
  * ---------------------
  * ServletUtilities.java
  * ---------------------
- * (C) Copyright 2002-2006, by Richard Atkinson and Contributors.
+ * (C) Copyright 2002-2007, by Richard Atkinson and Contributors.
  *
  * Original Author:  Richard Atkinson;
  * Contributor(s):   J?rgen Hoffman;
  *                   David Gilbert (for Object Refinery Limited);
  *                   Douglas Clayton;
  *
- * $Id: ServletUtilities.java,v 1.3.2.2 2006/01/10 15:53:30 mungady Exp $
- *
  * Changes
  * -------
  * 19-Aug-2002 : Version 1;
@@ -49,14 +47,15 @@
  *               by System property java.io.tmpdir, rather than default (RA);
  * 24-Mar-2004 : Added temp filename prefix attribute (DG);
  * 09-Mar-2005 : Added "one time" file option (DG);
- * ------------- JFREECHART 1.0.0 RELEASED ------------------------------------
+ * ------------- JFREECHART 1.0.x RELEASED ------------------------------------
  * 10-Jan-2006 : Updated API docs and reformatted (DG);
+ * 13-Sep-2006 : Format date in response header in English, not locale default
+ *               (see bug 1557141) (DG);
  *
  */
 
 package org.jfree.chart.servlet;
 
-
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.File;
@@ -65,6 +64,7 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.text.SimpleDateFormat;
 import java.util.Date;
+import java.util.Locale;
 import java.util.TimeZone;
 
 import javax.servlet.http.HttpServletResponse;
@@ -380,7 +380,7 @@ public class ServletUtilities {
             }
             response.setHeader("Content-Length", String.valueOf(file.length()));
             SimpleDateFormat sdf = new SimpleDateFormat(
-                    "EEE, dd MMM yyyy HH:mm:ss z");
+                    "EEE, dd MMM yyyy HH:mm:ss z", Locale.ENGLISH);
             sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
             response.setHeader("Last-Modified", 
                     sdf.format(new Date(file.lastModified())));
