@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,17 +27,17 @@
  * -----------------------------------
  * DefaultIntervalCategoryDataset.java
  * -----------------------------------
- * (C) Copyright 2002-2005, by Jeremy Bowman and Contributors.
+ * (C) Copyright 2002-2007, by Jeremy Bowman and Contributors.
  *
  * Original Author:  Jeremy Bowman;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *
- * $Id: DefaultIntervalCategoryDataset.java,v 1.9.2.2 2005/10/25 21:29:58 mungady Exp $
- *
  * Changes
  * -------
  * 29-Apr-2002 : Version 1, contributed by Jeremy Bowman (DG);
  * 24-Oct-2002 : Amendments for changes made to the dataset interface (DG);
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 08-Mar-2007 : Added equals() and clone() overrides (DG);
  *
  */
 
@@ -50,6 +50,7 @@ import java.util.List;
 import java.util.ResourceBundle;
 
 import org.jfree.data.DataUtilities;
+import org.jfree.data.UnknownKeyException;
 import org.jfree.data.general.AbstractSeriesDataset;
 
 /**
@@ -58,8 +59,6 @@ import org.jfree.data.general.AbstractSeriesDataset;
  * <p>
  * The standard constructor accepts data in a two dimensional array where the
  * first dimension is the series, and the second dimension is the category.
- *
- * @author Jeremy Bowman
  */
 public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
                                             implements IntervalCategoryDataset {
@@ -83,10 +82,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param ends  the ending values for the intervals.
      */
     public DefaultIntervalCategoryDataset(double[][] starts, double[][] ends) {
-        this(
-            DataUtilities.createNumberArray2D(starts),
-            DataUtilities.createNumberArray2D(ends)
-        );
+        this(DataUtilities.createNumberArray2D(starts),
+                DataUtilities.createNumberArray2D(ends));
     }
 
     /**
@@ -161,16 +158,15 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
 
                     if (seriesKeys.length != seriesCount) {
                         throw new IllegalArgumentException(
-                            "The number of series keys does "
-                            + "not match the number of series in the data."
-                        );
+                                "The number of series keys does not "
+                                + "match the number of series in the data.");
                     }
 
                     this.seriesKeys = seriesKeys;
                 }
                 else {
-                    String prefix 
-                        = resources.getString("series.default-prefix") + " ";
+                    String prefix = resources.getString(
+                            "series.default-prefix") + " ";
                     this.seriesKeys = generateKeys(seriesCount, prefix);
                 }
 
@@ -186,16 +182,14 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
                 if (categoryKeys != null) {
                     if (categoryKeys.length != categoryCount) {
                         throw new IllegalArgumentException(
-                            "The number of category keys does "
-                            + "not match the number of categories in the data."
-                        );
+                                "The number of category keys does not match "
+                                + "the number of categories in the data.");
                     }
                     this.categoryKeys = categoryKeys;
                 }
                 else {
                     String prefix = resources.getString(
-                        "categories.default-prefix"
-                    ) + " ";
+                            "categories.default-prefix") + " ";
                     this.categoryKeys = generateKeys(categoryCount, prefix);
                 }
 
@@ -212,6 +206,9 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * Returns the number of series in the dataset (possibly zero).
      *
      * @return The number of series in the dataset.
+     * 
+     * @see #getRowCount()
+     * @see #getCategoryCount()
      */
     public int getSeriesCount() {
         int result = 0;
@@ -221,25 +218,25 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
         return result;
     }
 
-    /**
-     * Returns the item count.
-     *
-     * @return The item count.
-     */
-    public int getItemCount() {
-        return this.categoryKeys.length;
-    }
-
     /**
      * Returns a series index.
      *
-     * @param series  the series key.
+     * @param seriesKey  the series key.
      *
      * @return The series index.
+     * 
+     * @see #getRowIndex(Comparable)
+     * @see #getSeriesKey(int)
      */
-    public int getSeriesIndex(Comparable series) {
-        List seriesKeys = getSeries();
-        return seriesKeys.indexOf(series);
+    public int getSeriesIndex(Comparable seriesKey) {
+        int result = -1;
+        for (int i = 0; i < this.seriesKeys.length; i++) {
+            if (seriesKey.equals(this.seriesKeys[i])) {
+                result = i;
+                break;
+            }
+        }
+        return result;
     }
 
     /**
@@ -248,6 +245,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param series  the index of the required series (zero-based).
      *
      * @return The name of the specified series.
+     * 
+     * @see #getSeriesIndex(Comparable)
      */
     public Comparable getSeriesKey(int series) {
         if ((series >= getSeriesCount()) || (series < 0)) {
@@ -259,34 +258,30 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
     /**
      * Sets the names of the series in the dataset.
      *
-     * @param seriesKeys  the keys of the series in the dataset.
+     * @param seriesKeys  the new keys (<code>null</code> not permitted, the 
+     *         length of the array must match the number of series in the 
+     *         dataset).
+     *         
+     * @see #setCategoryKeys(Comparable[])
      */
     public void setSeriesKeys(Comparable[] seriesKeys) {
-
-        // check argument...
         if (seriesKeys == null) {
             throw new IllegalArgumentException("Null 'seriesKeys' argument.");
         }
-
         if (seriesKeys.length != getSeriesCount()) {
             throw new IllegalArgumentException(
-                "DefaultIntervalCategoryDataset.setSeriesKeys(): "
-                + "the number of series keys does not match the data."
-            );
+                    "The number of series keys does not match the data.");
         }
-
-        // make the change...
         this.seriesKeys = seriesKeys;
         fireDatasetChanged();
-
     }
 
     /**
      * Returns the number of categories in the dataset.
-     * <P>
-     * This method is part of the CategoryDataset interface.
      *
      * @return The number of categories in the dataset.
+     * 
+     * @see #getColumnCount()
      */
     public int getCategoryCount() {
         int result = 0;
@@ -297,58 +292,25 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
         }
         return result;
     }
-
+    
     /**
-     * Returns a list of the series in the dataset.
-     * <P>
-     * Supports the CategoryDataset interface.
-     *
-     * @return A list of the series in the dataset.
-     */
-    public List getSeries() {
-
-        // the CategoryDataset interface expects a list of series, but
-        // we've stored them in an array...
-        if (this.seriesKeys == null) {
-            return new java.util.ArrayList();
-        }
-        else {
-            return Collections.unmodifiableList(Arrays.asList(this.seriesKeys));
-        }
-
-    }
-
-    /**
-     * Returns a list of the categories in the dataset.
-     * <P>
-     * Supports the CategoryDataset interface.
-     *
-     * @return A list of the categories in the dataset.
-     */
-    public List getCategories() {
-        return getColumnKeys();
-    }
-
-    /**
-     * Returns a list of the categories in the dataset.
-     * <P>
-     * Supports the CategoryDataset interface.
+     * Returns a list of the categories in the dataset.  This method supports 
+     * the {@link CategoryDataset} interface.
      *
      * @return A list of the categories in the dataset.
+     * 
+     * @see #getRowKeys()
      */
     public List getColumnKeys() {
-
         // the CategoryDataset interface expects a list of categories, but
         // we've stored them in an array...
         if (this.categoryKeys == null) {
             return new ArrayList();
         }
         else {
-            return Collections.unmodifiableList(
-                Arrays.asList(this.categoryKeys)
-            );
+            return Collections.unmodifiableList(Arrays.asList(
+                    this.categoryKeys));
         }
-
     }
 
     /**
@@ -356,20 +318,18 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      *
      * @param categoryKeys  an array of objects representing the categories in 
      *                      the dataset.
+     *                      
+     * @see #getRowKeys()
+     * @see #setSeriesKeys(Comparable[])
      */
     public void setCategoryKeys(Comparable[] categoryKeys) {
-
-        // check arguments...
         if (categoryKeys == null) {
             throw new IllegalArgumentException("Null 'categoryKeys' argument.");
         }
-
         if (categoryKeys.length != this.startData[0].length) {
             throw new IllegalArgumentException(
-                "The number of categories does not match the data."
-            );
+                    "The number of categories does not match the data.");
         }
-
         for (int i = 0; i < categoryKeys.length; i++) {
             if (categoryKeys[i] == null) {
                 throw new IllegalArgumentException(
@@ -377,11 +337,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
                     + "null category not permitted.");
             }
         }
-
-        // make the change...
         this.categoryKeys = categoryKeys;
         fireDatasetChanged();
-
     }
 
     /**
@@ -389,13 +346,23 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * <P>
      * This method is part of the CategoryDataset interface.  Not particularly
      * meaningful for this class...returns the end value.
+     * 
      * @param series    The required series (zero based index).
      * @param category  The required category.
+     * 
      * @return The data value for one category in a series (null possible).
+     * 
+     * @see #getEndValue(Comparable, Comparable)
      */
     public Number getValue(Comparable series, Comparable category) {
         int seriesIndex = getSeriesIndex(series);
+        if (seriesIndex < 0) {
+            throw new UnknownKeyException("Unknown 'series' key.");
+        }
         int itemIndex = getColumnIndex(category);
+        if (itemIndex < 0) {
+            throw new UnknownKeyException("Unknown 'category' key.");
+        }
         return getValue(seriesIndex, itemIndex);
     }
 
@@ -409,6 +376,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param category  the required category.
      *
      * @return The data value for one category in a series (null possible).
+     * 
+     * @see #getEndValue(int, int)
      */
     public Number getValue(int series, int category) {
         return getEndValue(series, category);
@@ -422,10 +391,18 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      *
      * @return The start data value for one category in a series 
      *         (possibly <code>null</code>).
+     *         
+     * @see #getStartValue(int, int)
      */
     public Number getStartValue(Comparable series, Comparable category) {
         int seriesIndex = getSeriesIndex(series);
+        if (seriesIndex < 0) {
+            throw new UnknownKeyException("Unknown 'series' key.");
+        }
         int itemIndex = getColumnIndex(category);
+        if (itemIndex < 0) {
+            throw new UnknownKeyException("Unknown 'category' key.");
+        }
         return getStartValue(seriesIndex, itemIndex);
     }
 
@@ -437,6 +414,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      *
      * @return The start data value for one category in a series 
      *         (possibly <code>null</code>).
+     *         
+     * @see #getStartValue(Comparable, Comparable)
      */
     public Number getStartValue(int series, int category) {
 
@@ -465,10 +444,18 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param category  the required category.
      *
      * @return The end data value for one category in a series (null possible).
+     * 
+     * @see #getEndValue(int, int)
      */
     public Number getEndValue(Comparable series, Comparable category) {
         int seriesIndex = getSeriesIndex(series);
+        if (seriesIndex < 0) {
+            throw new UnknownKeyException("Unknown 'series' key.");
+        }
         int itemIndex = getColumnIndex(category);
+        if (itemIndex < 0) {
+            throw new UnknownKeyException("Unknown 'category' key.");
+        }
         return getEndValue(seriesIndex, itemIndex);
     }
 
@@ -479,10 +466,10 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param category  the required category.
      *
      * @return The end data value for one category in a series (null possible).
+     * 
+     * @see #getEndValue(Comparable, Comparable)
      */
     public Number getEndValue(int series, int category) {
-
-        // check arguments...
         if ((series < 0) || (series >= getSeriesCount())) {
             throw new IllegalArgumentException(
                 "DefaultIntervalCategoryDataset.getValue(): "
@@ -495,9 +482,7 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
                 + "category index out of range.");
         }
 
-        // fetch the value...
         return this.endData[series][category];
-
     }
 
     /**
@@ -507,11 +492,13 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param category  the category.
      * 
      * @param value The value.
+     * 
+     * @see #setEndValue(int, Comparable, Number)
      */
     public void setStartValue(int series, Comparable category, Number value) {
 
         // does the series exist?
-        if ((series < 0) || (series > getSeriesCount())) {
+        if ((series < 0) || (series > getSeriesCount() - 1)) {
             throw new IllegalArgumentException(
                 "DefaultIntervalCategoryDataset.setValue: "
                 + "series outside valid range.");
@@ -538,11 +525,13 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param category  the category.
      *
      * @param value the value.
+     * 
+     * @see #setStartValue(int, Comparable, Number)
      */
     public void setEndValue(int series, Comparable category, Number value) {
 
         // does the series exist?
-        if ((series < 0) || (series > getSeriesCount())) {
+        if ((series < 0) || (series > getSeriesCount() - 1)) {
             throw new IllegalArgumentException(
                 "DefaultIntervalCategoryDataset.setValue: "
                 + "series outside valid range.");
@@ -565,11 +554,13 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
     /**
      * Returns the index for the given category.
      *
-     * @param category  the category.
+     * @param category  the category (<code>null</code> not permitted).
      *
      * @return The index.
+     * 
+     * @see #getColumnIndex(Comparable)
      */
-    private int getCategoryIndex(Comparable category) {
+    public int getCategoryIndex(Comparable category) {
         int result = -1;
         for (int i = 0; i < this.categoryKeys.length; i++) {
             if (category.equals(this.categoryKeys[i])) {
@@ -605,6 +596,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param column  the column index.
      *
      * @return The column key.
+     * 
+     * @see #getRowKey(int)
      */
     public Comparable getColumnKey(int column) {
         return this.categoryKeys[column];
@@ -613,13 +606,17 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
     /**
      * Returns a column index.
      *
-     * @param columnKey  the column key.
+     * @param columnKey  the column key (<code>null</code> not permitted).
      *
      * @return The column index.
+     * 
+     * @see #getCategoryIndex(Comparable)
      */
     public int getColumnIndex(Comparable columnKey) {
-        List categories = getCategories();
-        return categories.indexOf(columnKey);
+        if (columnKey == null) {
+            throw new IllegalArgumentException("Null 'columnKey' argument.");
+        }
+        return getCategoryIndex(columnKey);
     }
 
     /**
@@ -628,18 +625,20 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param rowKey  the row key.
      *
      * @return The row index.
+     * 
+     * @see #getSeriesIndex(Comparable)
      */
     public int getRowIndex(Comparable rowKey) {
-        List seriesKeys = getSeries();
-        return seriesKeys.indexOf(rowKey);
+        return getSeriesIndex(rowKey);
     }
 
     /**
-     * Returns a list of the series in the dataset.
-     * <P>
-     * Supports the CategoryDataset interface.
+     * Returns a list of the series in the dataset.  This method supports the 
+     * {@link CategoryDataset} interface.
      *
      * @return A list of the series in the dataset.
+     * 
+     * @see #getColumnKeys()
      */
     public List getRowKeys() {
         // the CategoryDataset interface expects a list of series, but
@@ -658,6 +657,8 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * @param row  the index of the required row/series (zero-based).
      *
      * @return The name of the specified series.
+     * 
+     * @see #getColumnKey(int)
      */
     public Comparable getRowKey(int row) {
         if ((row >= getRowCount()) || (row < 0)) {
@@ -672,28 +673,159 @@ public class DefaultIntervalCategoryDataset extends AbstractSeriesDataset
      * the {@link CategoryDataset} interface.
      *
      * @return The number of categories in the dataset.
+     * 
+     * @see #getCategoryCount()
+     * @see #getRowCount()
      */
     public int getColumnCount() {
-        int result = 0;
-        if (this.startData != null) {
-            if (getSeriesCount() > 0) {
-                result = this.startData[0].length;
-            }
-        }
-        return result;
+        return this.categoryKeys.length;
     }
 
     /**
      * Returns the number of series in the dataset (possibly zero).
      *
      * @return The number of series in the dataset.
+     * 
+     * @see #getSeriesCount()
+     * @see #getColumnCount()
      */
     public int getRowCount() {
-        int result = 0;
-        if (this.startData != null) {
-            result = this.startData.length;
+        return this.seriesKeys.length;
+    }
+    
+    /**
+     * Tests this dataset for equality with an arbitrary object.
+     * 
+     * @param obj  the object (<code>null</code> permitted).
+     * 
+     * @return A boolean.
+     */
+    public boolean equals(Object obj) {
+        if (obj == this) {
+            return true;
+        }
+        if (!(obj instanceof DefaultIntervalCategoryDataset)) {
+            return false;
+        }
+        DefaultIntervalCategoryDataset that 
+                = (DefaultIntervalCategoryDataset) obj;
+        if (!Arrays.equals(this.seriesKeys, that.seriesKeys)) {
+            return false;
+        }
+        if (!Arrays.equals(this.categoryKeys, that.categoryKeys)) {
+            return false;
+        }
+        if (!equal(this.startData, that.startData)) {
+            return false;
+        }
+        if (!equal(this.endData, that.endData)) {
+            return false;
+        }
+        // seem to be the same...
+        return true;
+    }
+
+    /**
+     * Returns a clone of this dataset.
+     * 
+     * @return A clone.
+     * 
+     * @throws CloneNotSupportedException if there is a problem cloning the
+     *         dataset.
+     */
+    public Object clone() throws CloneNotSupportedException {
+        DefaultIntervalCategoryDataset clone 
+                = (DefaultIntervalCategoryDataset) super.clone();
+        clone.categoryKeys = (Comparable[]) this.categoryKeys.clone();
+        clone.seriesKeys = (Comparable[]) this.seriesKeys.clone();
+        clone.startData = clone(this.startData);
+        clone.endData = clone(this.endData);
+        return clone;
+    }
+    
+    /**
+     * Tests two double[][] arrays for equality.
+     * 
+     * @param array1  the first array (<code>null</code> permitted).
+     * @param array2  the second arrray (<code>null</code> permitted).
+     * 
+     * @return A boolean.
+     */
+    private static boolean equal(Number[][] array1, Number[][] array2) {
+        if (array1 == null) {
+            return (array2 == null);
+        }
+        if (array2 == null) {
+            return false;
+        }
+        if (array1.length != array2.length) {
+            return false;
+        }
+        for (int i = 0; i < array1.length; i++) {
+            if (!Arrays.equals(array1[i], array2[i])) {
+                return false;
+            }
+        }
+        return true;
+    }
+    
+    /**
+     * Clones a two dimensional array of <code>Number</code> objects.
+     * 
+     * @param array  the array (<code>null</code> not permitted).
+     * 
+     * @return A clone of the array.
+     */
+    private static Number[][] clone(Number[][] array) {
+        if (array == null) {
+            throw new IllegalArgumentException("Null 'array' argument.");
+        }
+        Number[][] result = new Number[array.length][];
+        for (int i = 0; i < array.length; i++) {
+            Number[] child = array[i];
+            Number[] copychild = new Number[child.length];
+            System.arraycopy(child, 0, copychild, 0, child.length);
+            result[i] = copychild;
         }
         return result;
     }
 
+    /**
+     * Returns a list of the series in the dataset.
+     *
+     * @return A list of the series in the dataset.
+     * 
+     * @deprecated Use {@link #getRowKeys()} instead.
+     */
+    public List getSeries() {
+        if (this.seriesKeys == null) {
+            return new java.util.ArrayList();
+        }
+        else {
+            return Collections.unmodifiableList(Arrays.asList(this.seriesKeys));
+        }
+    }
+
+    /**
+     * Returns a list of the categories in the dataset.
+     *
+     * @return A list of the categories in the dataset.
+     * 
+     * @deprecated Use {@link #getColumnKeys()} instead.
+     */
+    public List getCategories() {
+        return getColumnKeys();
+    }
+
+    /**
+     * Returns the item count.
+     *
+     * @return The item count.
+     * 
+     * @deprecated Use {@link #getCategoryCount()} instead.
+     */
+    public int getItemCount() {
+        return this.categoryKeys.length;
+    }
+
 }
