@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,13 +27,11 @@
  * -------------------------
  * TaskSeriesCollection.java
  * -------------------------
- * (C) Copyright 2002-2006, by Object Refinery Limited.
+ * (C) Copyright 2002-2007, by Object Refinery Limited.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Thomas Schuster;
  *
- * $Id: TaskSeriesCollection.java,v 1.9.2.4 2006/01/26 14:51:20 mungady Exp $
- *
  * Changes
  * -------
  * 06-Jun-2002 : Version 1 (DG);
@@ -198,11 +196,14 @@ public class TaskSeriesCollection extends AbstractSeriesDataset
     /**
      * Returns the column index for a column key.
      *
-     * @param columnKey  the columnKey.
+     * @param columnKey  the column key (<code>null</code> not permitted).
      *
      * @return The column index.
      */
     public int getColumnIndex(Comparable columnKey) {
+        if (columnKey == null) {
+            throw new IllegalArgumentException("Null 'columnKey' argument.");
+        }
         return this.keys.indexOf(columnKey);
     }
 
@@ -291,7 +292,7 @@ public class TaskSeriesCollection extends AbstractSeriesDataset
      * @param series  the series (zero based index).
      */
     public void remove(int series) {
-        if ((series < 0) || (series > getSeriesCount())) {
+        if ((series < 0) || (series >= getSeriesCount())) {
             throw new IllegalArgumentException(
                 "TaskSeriesCollection.remove(): index outside valid range.");
         }
